/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// station list class 

#ifndef _STATION_CLASSES_
#define _STATION_CLASSES_

#include "defines.h"

class stations_cs
{
 public:
  // station and freq information is initialized as empty
  stations_cs ();
  // station and freq are set respectively
  stations_cs (const char [], const char []);
  // sets station and freq info for current record in list
  void set_info(const char [], const char []);
  // places station and freq info into char array
  void get_info(char *, char *);
  // records next station
  stations_cs *next;

 private:
  char station[MAX_STAT_INPUT+1];
  char freq[MAX_FREQ_INPUT+1];
};

class stations_list_cs
{
 public:
  // initializes list
  stations_list_cs ();
  // initializes list and adds a record
  stations_list_cs (const char [], const char []);
  // frees list (calls free_list)
  ~stations_list_cs ();
  // sets station counter for get_station_next to top of list
  int get_station_start ();
  // places next record into array
  int get_station_next  (char *, char *);
  // add a record
  int add_station (const char [], const char []);
  // free the list from memory
  void free_list();
  int get_num_recs ();

 private:
  stations_cs *HEAD, *CURRENT, LISTHEAD, *NEXTITEM;
  stations_cs *create_station(stations_cs *);
  void free_loop(stations_cs *);
  enum FREEDOM {YES = -1, NO};
  FREEDOM can_free;
  int total_recs;
};

#endif
