/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <gdk/gdkkeysyms.h>
#include <gdk_imlib.h>

#include "main.h"
#include "defines.h"
#include "callbacks.h"
#include "interface.h"
#include "radioclass.h"
#include "scanclass.h"
#include "externs.h"
#include "lirc.h"

GtkWidget *freq_new_win;
GtkWidget *new_freq_win_entry;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int
TUNE (char *fq)
{
  if (tha_dj.tune (fq) == RET_RNG) {
    MessageBox (tha_dj.freq_range);
    return 1;
  }

  strncpy (current_freq, fq, sizeof (current_freq));
  SetTextFreq (fq);
  return 0;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
on_close_dialog (GtkWidget *widget, gpointer gdata)
{
  gtk_widget_destroy (GTK_WIDGET (gdata));
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
gint
parse_key_press_new_freq (GtkWidget *widget, GdkEventKey *event)
{
  char freq[MAX_FREQ_INPUT+1];

  switch (event->keyval) {
  case GDK_Return:
    strcpy (freq, gtk_entry_get_text (GTK_ENTRY (new_freq_win_entry)));
    
    TUNE (freq);
  case GDK_Escape:
    gtk_widget_destroy (freq_new_win);
    break;
  }
  return 0;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
create_freq_new_win ()
{
    freq_new_win = gtk_window_new (GTK_WINDOW_POPUP);
    gtk_object_set_data (GTK_OBJECT (freq_new_win), "freq_new_win", freq_new_win);
    gtk_window_set_title (GTK_WINDOW (freq_new_win), "freq");
    gtk_window_set_position (GTK_WINDOW (freq_new_win), GTK_WIN_POS_MOUSE);
    gtk_window_set_modal (GTK_WINDOW (freq_new_win), TRUE);
    gtk_window_set_policy (GTK_WINDOW (freq_new_win), FALSE, FALSE, FALSE);
    
    new_freq_win_entry = gtk_entry_new_with_max_length (6);
    gtk_widget_ref (new_freq_win_entry);
    gtk_object_set_data_full (GTK_OBJECT (freq_new_win), "new_freq_win_entry",
			      new_freq_win_entry, (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (new_freq_win_entry);
    gtk_container_add (GTK_CONTAINER (freq_new_win), new_freq_win_entry);
    gtk_widget_set_usize (new_freq_win_entry, 50, -2);
    gtk_widget_show (freq_new_win);

    gtk_signal_connect (GTK_OBJECT (freq_new_win), "key_press_event",
			GTK_SIGNAL_FUNC (parse_key_press_new_freq), NULL);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
on_options_clicked (GtkButton *button, gpointer user_data)
{
  if (options_win_status == TOGGLED) { return; }

  options_win_status = TOGGLED;
  win_options = create_options_window ();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
on_load_freq_clicked (GtkWidget *entry,  gpointer user_data)
{
  char f[MAX_FREQ_INPUT+1];
  tha_dj.get_freq (f);
  gtk_entry_set_text (GTK_ENTRY (options_freq_choice_entry), f);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
on_optionmenu_clicked (GtkLabel *label, gpointer user_data)
{
  gchar *buff;
  char station[MAX_STAT_INPUT+1], freq[MAX_FREQ_INPUT+1];
  gtk_label_get (label, &buff);

  tha_list->get_station_start();
  // search for the station that matches the menu label
  while (tha_list->get_station_next (station, freq) != RET_EOL) {
    if (strcmp(station, buff) == 0) {
      TUNE(freq);
      break;
    }
  }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
on_options_optionmenu_clicked (GtkLabel *label, gpointer user_data)
{
  gchar *buff;
  char station[MAX_STAT_INPUT+1], freq[MAX_FREQ_INPUT+1];
  tha_list->get_station_start();

  gtk_label_get (label, &buff);

  options_stations_switched_toggle = TOGGLED;

  // search for the station that matches the menu label
  while (tha_list->get_station_next (station, freq) != RET_EOL) {
    if (strcmp(station, buff) == 0) {
      TUNE(freq);
      strcpy (options_station_choice_str_temp, station);
      gtk_option_menu_set_history (GTK_OPTION_MENU (win_main_menu),
				   get_list_position (station));
      break;
    }
  }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
on_mute_toggled_toggled (GtkToggleButton *toggleb, gpointer data)
{
  if (mute_is_on_toggle == UNTOGGLED) {
    tha_dj.mute();
    mute_is_on_toggle = TOGGLED;
    gtk_widget_destroy (GTK_BIN (toggle_mute_btn)->child);
    gtk_container_add (GTK_CONTAINER (toggle_mute_btn),
		       make_icon_button (win_main, PATH_MUTE_ON));
  } else {
    tha_dj.unmute();
    mute_is_on_toggle = UNTOGGLED;
    gtk_widget_destroy (GTK_BIN (toggle_mute_btn)->child);
    gtk_container_add (GTK_CONTAINER (toggle_mute_btn),
		       make_icon_button (win_main, PATH_MUTE));
  }
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
on_tv_toggled (GtkWidget *w, gpointer g)
{
  if (strlen (options_tv_exec_str) == 0) { return; }

  char buff[MAX_STAT_INPUT+10];

  if (tv_is_on_toggle == UNTOGGLED) {
    tv_is_on_toggle = TOGGLED;

    // turn off the radio before exec'n pgrm
    tha_dj.off (0); // 0 for don't mute, 1 for mute

    snprintf (buff, sizeof (buff), "%s &", options_tv_exec_str);
    system (buff);

    gtk_widget_destroy (GTK_BIN (toggle_tv_btn)->child);
    gtk_container_add (GTK_CONTAINER (toggle_tv_btn),
		       make_icon_button (win_main, PATH_TV_ON));
  } else {
    tv_is_on_toggle = UNTOGGLED;

    snprintf(buff, sizeof (buff), "killall %s", options_tv_exec_str);
    system (buff);

    usleep (250);

    tha_dj.get_freq (buff);
    TUNE (buff);

    gtk_widget_destroy (GTK_BIN (toggle_tv_btn)->child);
    gtk_container_add (GTK_CONTAINER (toggle_tv_btn),
		       make_icon_button (win_main, PATH_TV));
  }
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_mixer_clicked (GtkButton *button, gpointer user_data)
{
  char exec[100];
  sprintf (exec, "%s &", options_mixer_str);
  system (exec);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_freq_inc_clicked(GtkButton *, gpointer)
{
  tha_dj.incfreq(options_tune_increment);
  char fq[MAX_FREQ_INPUT+1];
  tha_dj.get_freq (fq);
  TUNE (fq);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_freq_dec_clicked(GtkButton *, gpointer)
{
  tha_dj.decfreq(options_tune_increment);
  char fq[MAX_FREQ_INPUT+1];
  tha_dj.get_freq (fq);
  TUNE (fq);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_station_radiobutton_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  options_station_toggle = gtk_toggle_button_get_active (togglebutton) ? TOGGLED : UNTOGGLED;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_freqradiobutton_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  options_freq_toggle = gtk_toggle_button_get_active (togglebutton) ? TOGGLED : UNTOGGLED;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_slad_toggled (GtkToggleButton *togglebutton,  gpointer user_data)
{
  options_slad = gtk_toggle_button_get_active (togglebutton) ? TOGGLED : UNTOGGLED;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_store_location_checkbutton_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  temp_store =  gtk_toggle_button_get_active (togglebutton) ? TOGGLED : UNTOGGLED;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_mute_exit_checkbutton_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  temp_mute =  gtk_toggle_button_get_active (togglebutton) ? TOGGLED : UNTOGGLED;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_wine_checkbutton_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  temp_wine =  gtk_toggle_button_get_active (togglebutton) ? TOGGLED : UNTOGGLED;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// option menu for selecting default radio
void on_radio_optionmenu_clicked (GtkLabel *label,  gpointer user_data)
{
  gchar *buff;
  char tmp[MAX_FREQ_INPUT+1];
  gtk_label_get (label, &buff);

  if (strcmp (buff, options_radio_card) == 0) return;

  strcpy (options_radio_card, buff);

  switch (tha_dj.open_card (options_radio_card)) {
  case RET_CARD:
  case RET_ERR:
    if (errno == ENODEV) MessageBox(STR_NO_DEV);
    if (errno == EBUSY)  MessageBox(STR_DEV_BUSY);
    break;
  case RET_OK:
    tha_dj.get_freq(tmp);
    TUNE(tmp);
    break;
  }
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
save_options (int gtk)
{
  FILE *rc;
  char rc_file[100], *station, *freq, st[MAX_STAT_INPUT+1], fq[MAX_FREQ_INPUT+1];
  int counter;

  sprintf (rc_file, "%s/"CfgFile, getenv("HOME"));

  // convert from entries to strings
  if (gtk) {
    strcpy (options_freq_choice,
	   gtk_entry_get_text (GTK_ENTRY(options_freq_choice_entry)));
    strcpy (options_tune_increment,
  	   gtk_entry_get_text (GTK_ENTRY(options_tune_inc_entry)));
    strcpy (options_scan_freq_start_str, 
	   gtk_entry_get_text (GTK_ENTRY(options_scan_freq_start_entry)));
    strcpy (options_scan_freq_stop_str, 
	   gtk_entry_get_text (GTK_ENTRY(options_scan_freq_stop_entry)));
    strcpy (options_tv_exec_str, 
	   gtk_entry_get_text (GTK_ENTRY(options_tv_exec_entry)));
    strcpy (options_mixer_str, 
	   gtk_entry_get_text (GTK_ENTRY(options_mixer_entry)));
    strcpy (options_station_choice_str, options_station_choice_str_temp);
  }

  options_wine = temp_wine;
  options_mute_exit = temp_mute;
  options_store_location = temp_store;

  if (temp_fg_changed) {
    temp_fg_changed = false;
    *fg = *temp_fg;
  }

  if (temp_bg_changed) {
    temp_bg_changed = false;
    *bg = *temp_bg;
  }

  strcpy (use_font, temp_font);

  // recreate linked list from clist
  if (gtk) {
    TUNE (current_freq); // apply the new font

    tha_list->free_list();

    // get the station names and freqs from clist
    for (counter = 0; counter < GTK_CLIST(options_clist)->rows; counter++) {
      gtk_clist_get_text(GTK_CLIST(options_clist), counter, 0, &freq);
      gtk_clist_get_text(GTK_CLIST(options_clist), counter, 1, &station);
      
      if (tha_list->add_station(station, freq) == RET_MEM) {
	MessageBox (STR_NUFF_MEM);
	break;
      }
    }
  }

  rc = fopen (rc_file, "w");
  if (!rc) {
    perror ("file");
  } else {
    fprintf(rc, "# generated by rdj\n");
    fprintf(rc, "%s", CFG_Sec_Preferences);
    if (options_station_toggle == TOGGLED && options_slad == UNTOGGLED) {
      fprintf (rc, "%s = %s\n", CFG_station, options_station_choice_str);
    } else if (options_freq_toggle == TOGGLED) {
      fprintf (rc, "#%s = %s\n", CFG_station, options_station_choice_str);
    } else if (options_slad == TOGGLED) {
      fprintf (rc, "%s = %s\n", CFG_station, current_station);
    }
    fprintf (rc, "%s = %s\n", CFG_freq,       options_freq_choice);
    fprintf (rc, "%s = %s\n", CFG_card,       options_radio_card);
    fprintf (rc, "%s = %i\n", CFG_slad,       options_slad);
    fprintf (rc, "%s = %i\n", CFG_mute,       options_mute_exit);
    fprintf (rc, "%s = %s\n", CFG_tune_inc,   options_tune_increment);
    fprintf (rc, "%s = %i\n", CFG_wine,       options_wine);
    fprintf (rc, "%s = %s\n", CFG_scan_start, options_scan_freq_start_str);
    fprintf (rc, "%s = %s\n", CFG_scan_stop,  options_scan_freq_stop_str);
    fprintf (rc, "%s = %s\n", CFG_tv_exec,    options_tv_exec_str);
    fprintf (rc, "%s = %s\n", CFG_mixer,      options_mixer_str);
    fprintf (rc, "%s = %i\n", CFG_foreground_r, fg->red);
    fprintf (rc, "%s = %i\n", CFG_foreground_g, fg->grn);
    fprintf (rc, "%s = %i\n", CFG_foreground_b, fg->blu);
    fprintf (rc, "%s = %i\n", CFG_background_r, bg->red);
    fprintf (rc, "%s = %i\n", CFG_background_g, bg->grn);
    fprintf (rc, "%s = %i\n", CFG_background_b, bg->blu);
    fprintf (rc, "%s = %i\n", CFG_remember_location, options_store_location);
    fprintf (rc, "%s = %s\n", CFG_font, use_font);
    fprintf (rc, "\n");
    fprintf (rc, "%s", CFG_Sec_Stations);
    fprintf (rc, "# Your selected stations\n");

    tha_list->get_station_start();
    while (tha_list->get_station_next(st, fq) != RET_EOL) {
      fprintf (rc, "%s = %s\n", st, fq);
    }
    fclose(rc);
  }

  // set the new colors in the frequency display
  set_colorz ();

  // recreate the main option menu
  if (options_stations_switched_toggle == TOGGLED && gtk) {
    gtk_widget_destroy (win_main_menu);
    // create the option menu
    win_main_menu = create_station_menu ();
    gtk_widget_ref (win_main_menu);
    gtk_object_set_data_full (GTK_OBJECT (win_main), "win_main_menu", win_main_menu,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_box_pack_start (GTK_BOX (hbox2), win_main_menu, FALSE, FALSE, 0);
    gtk_widget_set_usize(win_main_menu, -2, 21);
    gtk_widget_show (win_main_menu);

    gtk_option_menu_set_history (GTK_OPTION_MENU (win_main_menu),
				 get_list_position (current_station));
  }
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
on_options_apply_clicked (GtkWidget *w,  gpointer user_data)
{
  save_options (1);
  gtk_entry_set_text (GTK_ENTRY (font_entry), use_font);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_options_save_clicked (GtkWidget *window, gpointer user_data)
{
  save_options (1);
  options_win_status = UNTOGGLED;
  // close the options window
  gtk_widget_destroy (GTK_WIDGET (user_data));
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_close_options (GtkWidget *window,  gpointer user_data)
{
  strcpy (options_station_choice_str_temp, options_station_choice_str);

  gtk_option_menu_set_history (GTK_OPTION_MENU (win_main_menu),
			       get_list_position (current_station));

  options_win_status = UNTOGGLED;
  gtk_widget_destroy (GTK_WIDGET (user_data));
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_clist_select_row (GtkCList *lst, gpointer p)
{
  options_stations_switched_toggle = TOGGLED;

  // fill entries with row info
  gchar *text;

  // freq
  gtk_clist_get_text (lst, lst->focus_row, 0, &text);
  gtk_entry_set_text (GTK_ENTRY (options_new_freq_entry), text);

  // when they select a row in the options window, tune that frequency
  TUNE(text);
  
  // station name
  gtk_clist_get_text (lst, lst->focus_row, 1, &text);
  gtk_entry_set_text (GTK_ENTRY (options_new_station_entry), text);
  gtk_option_menu_set_history (GTK_OPTION_MENU (win_main_menu), get_list_position (text));
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_freqchart_add_clicked (GtkCList *clist, gpointer p)
{
  gchar station[MAX_STAT_INPUT+1], freq[MAX_FREQ_INPUT+1];

  strcpy(freq, gtk_entry_get_text (GTK_ENTRY (options_new_freq_entry)));
  strcpy(station, gtk_entry_get_text (GTK_ENTRY (options_new_station_entry)));

  if (strlen(freq) == 0)    { MessageBox(STR_NEED_FREQ); return; }
  if (strlen(station) == 0) { MessageBox(STR_NEED_NAME); return; }

  if (strchr(freq, '#') || strchr(station, '#')) { 
    MessageBox(STR_NO_POUNDS); 
    return; 
  }

  gchar *temp[2] = { freq, station };
  gtk_clist_append (GTK_CLIST(options_clist), temp);
  gtk_clist_select_row (GTK_CLIST (options_clist), GTK_CLIST(options_clist)->rows, 0);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_freqchart_update_clicked (GtkCList *lst, gpointer p)
{
  gchar station[MAX_STAT_INPUT+1], freq[MAX_FREQ_INPUT+1];

  strcpy(freq, gtk_entry_get_text (GTK_ENTRY (options_new_freq_entry)));
  strcpy(station, gtk_entry_get_text (GTK_ENTRY (options_new_station_entry)));

  if (strlen(freq) == 0)    { MessageBox(STR_NEED_FREQ); return; }
  if (strlen(station) == 0) { MessageBox(STR_NEED_NAME); return; }

  if (strchr(freq, '#') || strchr(station, '#')) {
    MessageBox(STR_NO_POUNDS);
    return; 
  }

  // update chart
  gtk_clist_set_text (GTK_CLIST(options_clist), GTK_CLIST (options_clist)->focus_row, 0, freq);
  gtk_clist_set_text (GTK_CLIST(options_clist), GTK_CLIST (options_clist)->focus_row, 1, station);
  gtk_clist_select_row (GTK_CLIST (options_clist), GTK_CLIST(options_clist)->focus_row, 0);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_freqchart_remove_clicked (GtkCList *lst, gpointer user_data)
{
  // remove entry info from list
  gtk_clist_remove (GTK_CLIST(options_clist), GTK_CLIST (options_clist)->focus_row);
  gtk_clist_select_row (GTK_CLIST (options_clist), GTK_CLIST(options_clist)->focus_row, 0);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_freqchart_mv_up_clicked (GtkCList *lst, gpointer user_data)
{
  gtk_clist_swap_rows (GTK_CLIST (options_clist), GTK_CLIST(options_clist)->focus_row,
		       GTK_CLIST(options_clist)->focus_row -1);
  gtk_clist_select_row (GTK_CLIST (options_clist), GTK_CLIST(options_clist)->focus_row, 0);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_freqchart_mv_down_clicked (GtkCList *lst, gpointer user_data)
{
  gtk_clist_row_move (GTK_CLIST (options_clist), GTK_CLIST(options_clist)->focus_row, 
		      GTK_CLIST(options_clist)->focus_row +1);
  gtk_clist_select_row (GTK_CLIST (options_clist), GTK_CLIST(options_clist)->focus_row, 0);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void on_scan_button_clicked (GtkButton *button,  gpointer user_data)
{
  char start[MAX_FREQ_INPUT+1], end[MAX_FREQ_INPUT+1], inc[MAX_FREQ_INPUT+1];
  char station[MAX_STAT_INPUT+1];

  strcpy (start, gtk_entry_get_text (GTK_ENTRY (options_scan_freq_start_entry)));
  strcpy (end,   gtk_entry_get_text (GTK_ENTRY (options_scan_freq_stop_entry)));
  strcpy (inc,   gtk_entry_get_text (GTK_ENTRY(options_tune_inc_entry)));

  if (strlen(start) == 0) { strcpy(start, FREQLOW);  }
  if (strlen(end) == 0)   { strcpy(end,   FREQHIGH); }

  // turn off the radio, so scan class can have it
  tha_dj.off(0);

  // scan
  scanr myscan;
  stations_list_cs news;
  myscan.scan(options_radio_card, start, end, inc, news);

  strcpy (station,"null");
  strcpy (start,  "null");

  // add the new stations to clist
  if (news.get_station_start())
    while (news.get_station_next (station, start) != RET_EOL) {
      gchar *temp[2] = { start, station };
      gtk_clist_append (GTK_CLIST(options_clist), temp);
    }

  gtk_notebook_set_page (GTK_NOTEBOOK (notebook1), PAGE_STATS);
  gtk_clist_select_row (GTK_CLIST (options_clist), 0, 0);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int
SetTextFreq (const char *freq)
{
  if (win_made == UNTOGGLED) return 0;

  GdkFont *freq_font;
  GdkRectangle update_rect;
  char nfq[MAX_FREQ_INPUT+1], station[MAX_STAT_INPUT+1], name[MAX_STAT_INPUT+7];

  // keep the current station name
  tha_list->get_station_start ();
  while (tha_list->get_station_next (station, nfq) != RET_EOL) {
    if (strcmp (freq, nfq) == 0) {
      strncpy (current_station, station, sizeof (current_station));
      break;
    }
  }

  snprintf (name, sizeof (name), "rdj - %s", current_station);
  gtk_window_set_title (GTK_WINDOW (win_main), name);

  pixmap = gdk_pixmap_new (drawing_area->window,
			   drawing_area->allocation.width,
			   drawing_area->allocation.height,
			   -1);

  freq_font = gdk_font_load (use_font);
  if (!freq_font) freq_font = gdk_font_load (DFT_FONT);

  gdk_draw_rectangle (pixmap, freq_bg_gc,
		      TRUE, 0, 0,
		      drawing_area->allocation.width,
		      drawing_area->allocation.height);

  if (strlen (freq) > 5) {
    gdk_draw_text (pixmap, freq_font, freq_fg_gc, 3, 14, freq, strlen(freq));
  } else if (strlen (freq) > 4) {
    gdk_draw_text (pixmap, freq_font, freq_fg_gc, 5, 14, freq, strlen(freq));
  } else if (strlen (freq)) {
    gdk_draw_text (pixmap, freq_font, freq_fg_gc, 10, 14, freq, strlen(freq));
  }

  if (lirc_works) {
    GdkGC *gc;
    GdkColor *c = (GdkColor *) g_malloc (sizeof (GdkColor));

    c->red = 0xffff; c->green = c->blue = 0x0000;
    gdk_color_alloc (gdk_colormap_get_system (), c);

    gc = gdk_gc_new (drawing_area->window);
    gdk_gc_set_foreground (gc, c);

    gdk_draw_arc (pixmap, gc, 0,
		  drawing_area->allocation.width - 6, drawing_area->allocation.height - 6,
		  4, 4, 0, (360*64));
    gdk_gc_destroy (gc);
  }

  update_rect.x = 0; update_rect.y = 0;
  update_rect.width  = drawing_area->allocation.width;
  update_rect.height = drawing_area->allocation.height;
  gtk_widget_draw (drawing_area, &update_rect);
  return 0;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int
get_list_position (const char *str)
{
  int n = 0;
  bool found = false;
  char freq[MAX_FREQ_INPUT+1], station[MAX_STAT_INPUT+1];
  tha_list->get_station_start ();
  while (tha_list->get_station_next (station, freq) != RET_EOL) {
    if (strcmp (station, str) == 0) { found = true; break; }
    n++;
  }
  return (found) ? n : -1;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
gint
parse_key_press (GtkWidget *widget, GdkEventKey *event)
{
  char freq[MAX_FREQ_INPUT+1], prev_freq[MAX_FREQ_INPUT+1],
    station[MAX_STAT_INPUT+1], prev_station[MAX_STAT_INPUT+1];

  tha_list->get_station_start ();
  while (tha_list->get_station_next (station, freq) != RET_EOL);
  strcpy (prev_freq, freq);
  strcpy (prev_station, station);
  tha_list->get_station_start ();

  switch (event->keyval) {
  case GDK_Up:
    while (1) {
      // if prev record was the top of the list, reset to bottom of list
      if (tha_list->get_station_next (station, freq) == RET_EOL) {
	tha_list->get_station_start ();
	while (tha_list->get_station_next (station, freq) != RET_EOL);
	tha_list->get_station_start ();
      }
    
      if (strcmp (station, current_station) == 0) {
	gtk_option_menu_set_history (GTK_OPTION_MENU (win_main_menu),
				     get_list_position (prev_station));
	TUNE (prev_freq);
	break;
      }
      strcpy (prev_freq, freq);
      strcpy (prev_station, station);
    }    
    break;
  case GDK_Down:  change_station_down (); break;
  case GDK_Right: on_freq_dec_clicked (NULL, NULL); break;
  case GDK_Left:  on_freq_inc_clicked (NULL, NULL); break;
  case 'r': case 'R': enable_lirc_cb (); TUNE (current_freq); break;
  case 'o': case 'O': on_options_clicked (NULL, NULL); break;
  case 'q': case 'Q': case GDK_Escape: ExitApp (); break;
  case 's': case 'S': on_mixer_clicked (NULL, NULL); break;
  case 'm': case 'M': case GDK_KP_Subtract: case '-':
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_mute_btn),
				  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle_mute_btn)) ? false : true);
    break;
  case 't': case 'T':
    if (options_tv_exec_str[0])
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_tv_btn),
				    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle_tv_btn)) ? false : true);
    break;
  case 'n': case 'N': create_freq_new_win ();
    break;
  }

  return FALSE;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
gint
key_press_close_win (GtkWidget *widget, GdkEventKey *event)
{
  gtk_widget_destroy (widget);
  return FALSE;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
change_station_up ()
{
  char freq[MAX_FREQ_INPUT+1], station[MAX_STAT_INPUT+1];
  char prev_freq[MAX_FREQ_INPUT+1] = "", prev_station[MAX_STAT_INPUT+1] = "";
  char prev_freq2[MAX_FREQ_INPUT+1] = "", prev_station2[MAX_STAT_INPUT+1] = "";

  if (!strlen (prev_freq)) { 
    strcpy (prev_freq, current_freq);
    strcpy (prev_station, current_station);
  }

  while (1) {
    // if prev record was the top of the list, reset to bottom of list
    if (tha_list->get_station_next (station, freq) == RET_EOL) {
      tha_list->get_station_start ();
      while (tha_list->get_station_next (station, freq) != RET_EOL) {
	strcpy (prev_station2, prev_station); strcpy (prev_station, station);
	strcpy (prev_freq2, prev_freq); strcpy (prev_freq, freq);
      }

      strcpy (prev_freq, prev_freq2); strcpy (prev_station, prev_station2);
      tha_list->get_station_start ();
    }
    
    if (strcmp (station, current_station) == 0) {
      gtk_option_menu_set_history (GTK_OPTION_MENU (win_main_menu),
  				   get_list_position (prev_station));
      TUNE (prev_freq);
      break;
    }
    strcpy (prev_freq, freq);
    strcpy (prev_station, station);
  }
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
change_station_down ()
{
  char freq[MAX_FREQ_INPUT+1], station[MAX_STAT_INPUT+1];
  bool its_next = false;

  while (1) {
    if (tha_list->get_station_next (station, freq) == RET_EOL) {
      tha_list->get_station_start ();
      if (tha_list->get_station_next (station, freq) == RET_EOL) break;
    }
    
    if (strcmp (station, current_station) == 0) {
      its_next = true;
      continue;
    }
    if (its_next) {
      gtk_option_menu_set_history (GTK_OPTION_MENU (win_main_menu),
				   get_list_position (station));
      TUNE (freq);
      break;
    }
  }
}
