/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "defines.h"
#include "stationlist.h"
#include "config.h"
#define NO_GTK
#include "externs.h"

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// load the preferences and radio station listings
int ReadConfig()
{
   FILE *rc;
   char buff[100], left[MAX_STAT_INPUT+1], right[257];
   char rc_file[100];
   char station[MAX_STAT_INPUT+1], freq[MAX_FREQ_INPUT+1];
   int  section = 1, i, found, found2, is_station = 0;

   sprintf (rc_file, "%s/"CfgFile, getenv("HOME"));
   strcpy (left, "");
   strcpy (right, "");
   strcpy (options_mixer_str, "");
   strcpy (options_radio_card, "");
   strcpy (options_freq_choice, "");
   strcpy (options_tv_exec_str, "");
   strcpy (options_tune_increment, "");
   strcpy (options_scan_freq_stop_str, "");
   strcpy (options_station_choice_str, "");
   strcpy (options_scan_freq_start_str, "");

   rc = fopen (rc_file, "r");
   if (!rc) {
     rc = fopen (rc_file, "w");
     if (!rc) {
       printf (STR_NO_RC_CREATE": %s\n", rc_file);
       return RET_ERR;
     } else {
       fprintf (rc, "# generated by rdj\n");
       fprintf (rc, "%s", CFG_Sec_Preferences);
#ifdef italiano
       fprintf (rc, "%s = Bum Bum Energy\n", CFG_station);
       fprintf (rc, "%s = 90.4\n",  CFG_freq);
#endif
       fprintf (rc, "%s = "DFT_CARD"\n", CFG_card);
       fprintf (rc, "%s = 0\n",   CFG_mute);
       fprintf (rc, "%s = 0.2\n", CFG_tune_inc);
       fprintf (rc, "%s = 1\n",   CFG_wine);
       fprintf (rc, "%s = %s\n" , CFG_scan_start, FREQLOW);
       fprintf (rc, "%s = %s\n",  CFG_scan_stop,  FREQHIGH);
       fprintf (rc, "%s = xawtv\n", CFG_tv_exec);
       fprintf (rc, "%s = gmix\n", CFG_mixer);
       fprintf (rc, "%s = 1\n", CFG_remember_location);
       fprintf (rc, "%s = %s\n", CFG_font, use_font);
       fprintf (rc, "%s = %i\n", CFG_foreground_r, fg->red);
       fprintf (rc, "%s = %i\n", CFG_foreground_g, fg->grn);
       fprintf (rc, "%s = %i\n", CFG_foreground_b, fg->blu);
       fprintf (rc, "%s = %i\n", CFG_background_r, bg->red);
       fprintf (rc, "%s = %i\n", CFG_background_g, bg->grn);
       fprintf (rc, "%s = %i\n", CFG_background_b, bg->blu);
       fprintf (rc, "\n");
       fprintf (rc, "%s", CFG_Sec_Stations);
#ifdef italiano
       fprintf (rc, "# My favorite italian stations\n");
       fprintf (rc, "Star           = 102.75");
       fprintf (rc, "Gemini         = 96.0");
       fprintf (rc, "Bakano         = 97.45");
       fprintf (rc, "Deejay         = 90.2");
       fprintf (rc, "Fantasy        = 91.3");
       fprintf (rc, "Company        = 90.85");
       fprintf (rc, "Bum Bum Nrg    = 90.4");
       fprintf (rc, "Bum Bum Nwk    = 104.2");
       fprintf (rc, "Cento Cinque   = 87.7");
       fprintf (rc, "Italia Network = 96.76");
#endif
       fclose (rc);
       printf (STR_RC_CREATED": %s\n", rc_file);
     }
     rc = fopen (rc_file, "r");
   }
   
   // get the configuration
   while (fgets (buff, sizeof (buff), rc)) {
     strcpy (left, "");
     strcpy (right, "");
     found = 0;
     found2 = 0;

     // for values of the preferences section
     if (strcmp (buff, CFG_Sec_Preferences) == 0) {
       section = 1;
       continue;
     }
     
     // for capturing stations
     if (strcmp (buff, CFG_Sec_Stations) == 0) {
       section = 2;
       continue;
     }

     // if it's obvious this line is a comment
     for (i = 0; (unsigned) i < strlen(buff); i++) {
       if (buff[i] == '=') { found2 = 1; }
       if (buff[i] == '#' && found2 != 1) {
	 found = 1;
	 break;
       }
       if (found2 == 1) { break; }
     }
     if (found == 1) { continue; }
     
     //if it's an equation get it's values
     if (strchr(buff, '=')) { SplitEquation(buff, left, right); }
     else { continue; }
     
     switch(section) {
     case 1:   //parse the preferences
       //check the station
       if (strcmp(left, CFG_station) == 0) {
	 strncpy (options_station_choice_str, right, sizeof (options_station_choice_str));
       }
       
       //check the freq
       if (strcmp(left, CFG_freq) == 0 && strlen(right) != 0) {
	 strncpy (options_freq_choice, right, sizeof (options_freq_choice));
       }
       
       //check the card
       if (strcmp(left, CFG_card) == 0 && strlen(right) != 0) {
	 strncpy (options_radio_card, right, sizeof (options_radio_card));
       }

       //check the tune inc
       if (strcmp(left, CFG_tune_inc) == 0 && strlen(right) != 0) {
	 strncpy (options_tune_increment, right, sizeof (options_tune_increment));
       }
       
       //check the slad
       if (strcmp(left, CFG_slad) == 0 && strlen(right) != 0) {
	 if (right[0] != '1') { options_slad = UNTOGGLED;
	 } else { options_slad = TOGGLED; }
       }

       //check the mute
       if (strcmp(left, CFG_mute) == 0 && strlen(right) != 0) {
	 if (right[0] == '0') {
	   options_mute_exit = UNTOGGLED;
	   mute_on_exit = UNTOGGLED;
	   temp_mute = UNTOGGLED;
//  	   printf ("NO mute on exit\n");
	 } else {
	   options_mute_exit = TOGGLED;
	   mute_on_exit = TOGGLED;
	   temp_mute = TOGGLED;
//  	   printf ("MUTE on exit\n");
	 }
       }

       //check the wine
       if (strcmp(left, CFG_wine) == 0 && strlen(right) != 0) {
	 if (right[0] != '1') {
	   options_wine = UNTOGGLED;
	   temp_wine = UNTOGGLED;
	 } else {
	   options_wine = TOGGLED;
	   temp_wine = TOGGLED;
	 }
       }

       //check the scan start
       if (strcmp(left, CFG_scan_start) == 0 && strlen(right) != 0) {
	 strncpy (options_scan_freq_start_str, right, sizeof (options_scan_freq_start_str));
       }

       //check the scan stop
       if (strcmp(left, CFG_scan_stop) == 0 && strlen(right) != 0) {
	 strncpy (options_scan_freq_stop_str, right, sizeof (options_scan_freq_stop_str));
       }

       //check the tv exec
       if (strcmp(left, CFG_tv_exec) == 0 && strlen(right) != 0) {
	 strncpy (options_tv_exec_str, right, sizeof (options_tv_exec_str));
       }

       //check the mixer
       if (strcmp(left, CFG_mixer) == 0 && strlen(right) != 0) {
	 strncpy (options_mixer_str, right, sizeof (options_mixer_str));
       }

       //check the location store
       if (strcmp(left, CFG_remember_location) == 0 && strlen(right) != 0) {
	 if (right[0] != '0') {
	   options_store_location = TOGGLED;
	   temp_store = TOGGLED;
	 } else {
	   options_store_location = UNTOGGLED;
	   temp_store = UNTOGGLED;
	 }
       }

       //check the font
       if (strcmp(left, CFG_font) == 0 && strlen (right) != 0) {
	 strncpy (use_font, right, sizeof (use_font));
	 strncpy (temp_font, right, sizeof (temp_font));
       }

       //check the color foreground
       if (strcmp (left, CFG_foreground_r) == 0 && strlen (right) != 0) {
	 fg->red = atoi (right);
       }
       if (strcmp (left, CFG_foreground_g) == 0 && strlen (right) != 0) {
	 fg->grn = atoi (right);
       }
       if (strcmp (left, CFG_foreground_b) == 0 && strlen (right) != 0) {
	 fg->blu = atoi (right);
       }

       //check the color background
       if (strcmp (left, CFG_background_r) == 0 && strlen (right) != 0) {
	 bg->red = atoi (right);
       }
       if (strcmp (left, CFG_background_g) == 0 && strlen (right) != 0) {
	 bg->grn = atoi (right);
       }
       if (strcmp (left, CFG_background_b) == 0 && strlen (right) != 0) {
	 bg->blu = atoi (right);
       }
       break;   //end if section 1
     case 2:  //capture the stations
       // if no freq
       if (strlen(right) == 0) { break; }       
       
       // only allow MAX_STAT_INPUT digits per station
       if (strlen(left) > MAX_STAT_INPUT) {
	 for (i = 0; i < MAX_STAT_INPUT; i++) {
	   station[i] = right[i];
	 }
	 station[i] = '\0';
       } else {
	 strcpy(station, left);
       }

       // only allow five digits per freq
       if (strlen(right) > MAX_FREQ_INPUT) {
	 for (i = 0; i < MAX_FREQ_INPUT; i++) {
	   freq[i] = right[i];
	 }
	 freq[i] = '\0';
       } else {
	 strcpy(freq, right);
       }

       if (tha_list->add_station(station, freq) == RET_MEM) {
	 printf(STR_NUFF_MEM"\n");
	 //exit app
	 return RET_MEM;
       }

       is_station++;

       break;  //end if section 2
     }  //end of switch
   }  //end while

   if (is_station == 0) {
       if (tha_list->add_station (STR_NO_STATIONS, FREQLOW) == RET_MEM) {
	 printf (STR_NUFF_MEM"\n");
	 return RET_MEM;
       }
   }

   fclose(rc);
   return RET_OK;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void SplitEquation(char eq[], /* IN */
		   char left[], char right[] /* OUT */)
{
  int i = 0;

  // return on no equation
  if (strchr(eq, '=') == 0) {
    strcpy(left, "");
    strcpy(right, "");
    return;
  }

  // if no comment in line
  if (strchr(eq, '#') == 0) {
    strcpy(left, strtok(eq, "="));
    strcpy(right, strtok(NULL, "\0"));
  } else {
    strcpy(left, strtok(eq, "="));
    strcpy(right, strtok(NULL, "#"));
  }

  // get rid of space after last char in string
  while(left[strlen(left)-1] == ' ') {
    left[strlen(left)-1] = '\0';
  }

  // remove space before character
  do {
    if (left[0] == ' ') {
      for (i = 0; (unsigned) i < strlen(left); i++) {
	left[i] = left[i+1];
      }
    }
  } while (left[0] == ' ');

  // get rid of space after last char in string
  while(right[strlen(right)-1] == ' ') {
    right[strlen(right)-1] = '\0';
  }

  // remove space before character
  do {
    if (right[0] == ' ') {
      for (i = 0; (unsigned) i < strlen(right); i++) {
	right[i] = right[i+1];
      }
    }
  } while (right[0] == ' ');

  if (right[strlen(right)-1] == '\n') {
    right[strlen(right)-1] = '\0';
  }

  memset(eq, '\0', sizeof(eq));
}
