/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <gnome.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include "main.h"
#include "defines.h"
#include "callbacks.h"
#include "interface.h"
#include "radioclass.h"
#include "scanclass.h"
#include "externs.h"

#define DSIZE 35

GtkTooltips *tooltip;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
static void
color_set_fg_cb (GnomeColorPicker *cp, guint pr, guint pg, guint pb)
{
  temp_fg->red = pr;
  temp_fg->grn = pg;
  temp_fg->blu = pb;
  temp_fg_changed = true;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
static void
color_set_bg_cb (GnomeColorPicker *cp, guint pr, guint pg, guint pb)
{
  temp_bg->red = pr;
  temp_bg->grn = pg;
  temp_bg->blu = pb;
  temp_bg_changed = true;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
static void
font_ok_clicked_cb(GtkWidget *widget, GtkWidget *fs)
{
  strncpy (temp_font, g_strdup (gtk_font_selection_dialog_get_font_name (GTK_FONT_SELECTION_DIALOG (fs))), sizeof (temp_font));
  gtk_widget_destroy(fs);
}

static void
font_cancel_clicked_cb(GtkWidget *widget, GtkWidget *fs)
{
  gtk_widget_destroy (fs);
}       
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
static void
font_button_cb (GtkWidget *widget, gpointer *data)
{
  GtkWidget *fs;
  
  fs = gtk_font_selection_dialog_new ("  "STR_FONT_SELECTOR"  ");
  gtk_font_selection_dialog_set_font_name (GTK_FONT_SELECTION_DIALOG(fs), temp_font);
  
  gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(fs)->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(font_ok_clicked_cb), fs);
  gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(fs)->cancel_button), "clicked",
		     GTK_SIGNAL_FUNC(font_cancel_clicked_cb), fs);
  
  gtk_widget_show(fs);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
gint
expose_event (GtkWidget *widget, GdkEventExpose *event)
{
  if (win_made == UNTOGGLED) return FALSE;

  gdk_draw_pixmap (widget->window,
		   widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		   pixmap,
		   event->area.x, event->area.y,
		   event->area.x, event->area.y,
		   event->area.width, event->area.height);
  return FALSE;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
GtkWidget*
make_icon_button (GtkWidget *parent, gchar *xpm_filename)
{
  GtkWidget *pixmapwid;
  GdkPixmap *pixmap1;
  GdkBitmap *mask;
  GtkStyle  *style;

  style = gtk_widget_get_style (parent);
  gtk_widget_realize (parent);
  
  if ((pixmap1 = gdk_pixmap_create_from_xpm (parent->window, &mask,
					     &style->bg[GTK_STATE_NORMAL],
					     xpm_filename)) == NULL) {
    g_print ("error: can't access %s\n", xpm_filename);
    return gtk_label_new ("xpm");
  }
  pixmapwid = gtk_pixmap_new (pixmap1, mask);
  
  gtk_widget_show (pixmapwid);
  return pixmapwid;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
gint
darea_expose_event (GtkWidget *w, GdkEventExpose *event)
{
  GdkPixmap *p;

  p = gdk_pixmap_new (w->window,
		      w->allocation.width,
		      w->allocation.height,
		      -1);

  gdk_draw_rectangle (p, w->style->black_gc,
		      TRUE, 0, 0,
		      w->allocation.width,
		      w->allocation.height);

  gdk_draw_pixmap (w->window,
		   w->style->fg_gc[GTK_WIDGET_STATE (w)],
		   p,
		   event->area.x, event->area.y,
		   event->area.x, event->area.y,
		   event->area.width, event->area.height);
  return FALSE;
}



//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void MessageBox(const char *message)
{
  GtkWidget *msgbox, *vbox1, *msg, *button1, *hbuttonbox1;

  msgbox = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_title (GTK_WINDOW (msgbox), STR_RDJ_MESSAGE);
  gtk_window_set_policy (GTK_WINDOW (msgbox), FALSE, FALSE, FALSE);
  gtk_window_set_position (GTK_WINDOW (msgbox), GTK_WIN_POS_CENTER);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (msgbox), vbox1);

  msg = gtk_label_new (message);
  gtk_misc_set_padding (GTK_MISC (msg), 20, 10);
  gtk_widget_show (msg);
  gtk_box_pack_start (GTK_BOX (vbox1), msg, FALSE, FALSE, 0);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, FALSE, 0);

  button1 = gtk_button_new_with_label (STR_CLOSE);
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbuttonbox1), button1, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (on_close_dialog),
                      msgbox);
  gtk_signal_connect (GTK_OBJECT (msgbox), "key_press_event",
                      GTK_SIGNAL_FUNC (key_press_close_win), msgbox);

  gtk_widget_show(msgbox);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
GtkWidget *create_station_menu ()
{
  GtkWidget *option_menu, *option_menu_menu;
  GtkWidget *label;
  GtkWidget *menuitem;
  char station[MAX_STAT_INPUT+1], freq[MAX_FREQ_INPUT+1];

  option_menu = gtk_option_menu_new ();
  option_menu_menu = gtk_menu_new ();

  tha_list->get_station_start();
  while (tha_list->get_station_next(station, freq) != RET_EOL) {
    label = gtk_label_new (station);
    menuitem = gtk_menu_item_new ();
    gtk_container_add (GTK_CONTAINER (menuitem), label);
    gtk_widget_show (label);
    gtk_widget_show (menuitem);
    gtk_menu_append (GTK_MENU (option_menu_menu), menuitem);

    gtk_signal_connect_object (GTK_OBJECT (menuitem), "activate",
			       GTK_SIGNAL_FUNC (on_optionmenu_clicked),
			       GTK_OBJECT(label));
  }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), option_menu_menu);
  return option_menu;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
GtkWidget *create_options_station_menu ()
{
  GtkWidget *option_menu, *option_menu_menu;
  GtkWidget *label;
  GtkWidget *menuitem;
  char station[MAX_STAT_INPUT+1], freq[MAX_FREQ_INPUT+1];

  option_menu = gtk_option_menu_new ();
  option_menu_menu = gtk_menu_new ();

  options_stations_switched_toggle = UNTOGGLED;
  strcpy(options_station_choice_str_temp, options_station_choice_str);

  // if default freq is toggled in the resource file and then the user
  // toggles station as default the first station will be defaulted
  // if not otherwise selected by the user.
  tha_list->get_station_start();
  if (options_station_toggle == UNTOGGLED &&
      tha_list->get_station_next(station, freq) != RET_EOL) {
      strcpy(options_station_choice_str_temp, station);
  }

  // put the other stations in the menu
  tha_list->get_station_start();
  while (tha_list->get_station_next(station, freq) != RET_EOL &&
	 strlen(options_station_choice_str_temp)) {
    label = gtk_label_new (station);
    menuitem = gtk_menu_item_new ();
    gtk_container_add (GTK_CONTAINER (menuitem), label);
    gtk_widget_show (label);
    gtk_widget_show (menuitem);
    gtk_menu_append (GTK_MENU (option_menu_menu), menuitem);

    gtk_signal_connect_object (GTK_OBJECT (menuitem), "activate",
			       GTK_SIGNAL_FUNC (on_options_optionmenu_clicked),
			       GTK_OBJECT(label));
  }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), option_menu_menu);

  return option_menu;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
GtkWidget *create_win_main ()
{
  GtkWidget *win;
  GtkWidget *frame2;
  GtkWidget *vbox1, *vbox2;
  GtkWidget *hbox1;
  GtkWidget *button, *box;
  GtkWidget *togglebutton1;

  win = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_title (GTK_WINDOW (win), "rdj");
  gtk_window_set_policy (GTK_WINDOW (win), FALSE, FALSE, FALSE);
  gtk_signal_connect (GTK_OBJECT (win), "delete_event",
                      GTK_SIGNAL_FUNC (ExitApp), NULL);
  gtk_signal_connect (GTK_OBJECT (win), "key_press_event",
                      GTK_SIGNAL_FUNC (parse_key_press), NULL);

  tooltip = gtk_tooltips_new ();

  // main vbox for window
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (win), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (win), vbox1);

  // box contains drawing area, two buttons, and menu
  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (win), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, FALSE, 0);

  // drawing area
  drawing_area = gtk_drawing_area_new ();
  gtk_widget_ref (drawing_area);
  gtk_object_set_data_full (GTK_OBJECT (win), "drawing_area", drawing_area,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), 50, 18);
  gtk_box_pack_start (GTK_BOX (hbox2), drawing_area, FALSE, FALSE, 0);
  gtk_widget_show (drawing_area);
  gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",
                      (GtkSignalFunc) expose_event, NULL);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (win), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox2), vbox2, FALSE, FALSE, 0);

  button = gtk_button_new_with_label (".");
  gtk_widget_ref (button);
  gtk_object_set_data_full (GTK_OBJECT (win), "button", button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
  gtk_widget_set_usize (button, -2, 9);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_freq_inc_clicked),
                      NULL);
  gtk_widget_show (button);

  button = gtk_button_new_with_label (".");
  gtk_widget_ref (button);
  gtk_object_set_data_full (GTK_OBJECT (win), "button", button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
  gtk_widget_set_usize(button, -2, 9);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_freq_dec_clicked),
                      NULL);
  gtk_widget_show (button);

  // create the option menu
  win_main_menu = create_station_menu ();
  gtk_widget_ref (win_main_menu);
  gtk_object_set_data_full (GTK_OBJECT (win), "win_main_menu", win_main_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox2), win_main_menu, FALSE, FALSE, 0);
  gtk_widget_set_usize(win_main_menu, -2, 21);
  gtk_widget_show (win_main_menu);
  if (options_station_choice_str)
    gtk_option_menu_set_history (GTK_OPTION_MENU (win_main_menu),
				 get_list_position (options_station_choice_str));

  // second row of the main vbox
  frame2 = gtk_frame_new (NULL);
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (win), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (win), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (frame2), hbox1);

  // OPTIONS BUTTON
  box = make_icon_button (win, PATH_PREFS);
  button = gtk_button_new ();
  gtk_widget_ref (button);
  gtk_object_set_data_full (GTK_OBJECT (win), "button", button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (button), 3);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_options_clicked), NULL);
  gtk_container_add (GTK_CONTAINER (button), box);
  gtk_tooltips_set_tip (tooltip, button, STR_TIP_OPTIONS" [o]", NULL);
  gtk_widget_set_usize (button, 35, -2);
  gtk_widget_show (button);

  // MUTE BUTTON
  box = make_icon_button (win, PATH_MUTE);
  togglebutton1 = gtk_toggle_button_new ();
  gtk_widget_ref (togglebutton1);
  gtk_object_set_data_full (GTK_OBJECT (win), "togglebutton1", togglebutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox1), togglebutton1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (togglebutton1), 3);
  gtk_signal_connect (GTK_OBJECT (togglebutton1), "toggled",
                      GTK_SIGNAL_FUNC (on_mute_toggled_toggled), NULL);
  gtk_container_add (GTK_CONTAINER (togglebutton1), box);
  gtk_tooltips_set_tip (tooltip, togglebutton1, STR_TIP_MUTE" [m]", NULL);
  gtk_widget_show (togglebutton1);
  gtk_widget_set_usize (togglebutton1, 35, -2);
  toggle_mute_btn = togglebutton1;

  // MIXER BUTTON
  box = make_icon_button (win, PATH_MIXER);
  button = gtk_button_new ();
  gtk_widget_ref (button);
  gtk_object_set_data_full (GTK_OBJECT (win), "button", button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (button), 3);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_mixer_clicked), NULL);
  gtk_container_add (GTK_CONTAINER (button), box);
  gtk_tooltips_set_tip (tooltip, button, STR_TIP_MIXER" [s]", NULL);
  gtk_widget_set_usize (button, 35, -2);
  gtk_widget_show (button);

  // TV BUTTON
  if (options_tv_exec_str[0]) {
    box = make_icon_button (win, PATH_TV);
    togglebutton1 = gtk_toggle_button_new ();
    gtk_widget_ref (togglebutton1);
    gtk_object_set_data_full (GTK_OBJECT (win), "togglebutton1", togglebutton1,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_box_pack_start (GTK_BOX (hbox1), togglebutton1, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (togglebutton1), 3);
    gtk_signal_connect (GTK_OBJECT (togglebutton1), "toggled",
			GTK_SIGNAL_FUNC (on_tv_toggled), NULL);
    gtk_container_add (GTK_CONTAINER (togglebutton1), box);
    gtk_tooltips_set_tip (tooltip, togglebutton1, STR_TIP_TV" [t]", NULL);
    gtk_widget_set_usize (togglebutton1, 35, -2);
    gtk_widget_show (togglebutton1);
    toggle_tv_btn = togglebutton1;
  } else toggle_tv_btn = NULL;

  // EXIT BUTTON
  box = make_icon_button (win, PATH_EXIT);
  button = gtk_button_new ();
  gtk_widget_ref (button);
  gtk_object_set_data_full (GTK_OBJECT (win), "button", button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (button), 3);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (ExitApp),
                      NULL);
  gtk_container_add (GTK_CONTAINER (button), box);
  gtk_tooltips_set_tip (tooltip, button, STR_TIP_QUIT" [q]", NULL);
  gtk_widget_set_usize (button, 35, -2);
  gtk_widget_show (button);

  gtk_widget_show (hbox1);
  gtk_widget_show (hbox2);
  gtk_widget_show (vbox1);
  gtk_widget_show (vbox2);
  gtk_widget_show (frame2);
  gtk_widget_show (win);

  // for the colors
  freq_fg_gc = gdk_gc_new (win->window);
  gdk_gc_set_foreground (freq_fg_gc, freqdisp_fg);

  freq_bg_gc = gdk_gc_new (win->window);
  gdk_gc_set_foreground (freq_bg_gc, freqdisp_bg);

  return win;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
GtkWidget *create_options_window (void)
{
  int i;
  char buff[MAX_STAT_INPUT+1];
  GtkWidget *menuitem;
  GtkWidget *options_window;
  GtkWidget *scrolledwindow1;
  GSList *startup_sound_group = NULL;
  GtkWidget *checkbutton1, *checkbutton2, *hbuttonbox2;
  GtkWidget *label11, *label14, *label15, *label6;
  GtkWidget *hbuttonbox1, *hbuttonbox3, *hbuttonbox4;
  GtkWidget *vbox1, *vbox2, *vbox3, *vbox4, *vbox5, *vbox7, *vbox8, *about_vbox;
  GtkWidget *frame1, *frame2, *frame3, *frame4, *frame5, *frame6, *frame7, *frame8;
  GtkWidget *button1, *button2, *button4, *button5, *button6, *button7, *button8, *icon_box;
  GtkWidget *radiobutton1, *radiobutton2, *radio_optionmenu, *radio_optionmenu_menu;
  GtkWidget *label1, *label2, *label3, *label4, *label5, *label7, *label8, *label9, *label10;
  GtkWidget *hbox2, *hbox3, *hbox4, *hbox5, *hbox6, *hbox7, *hbox9, *hbox10, *hbox13;
  GtkWidget *fg_cp, *bg_cp;

  options_window = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_object_set_data (GTK_OBJECT (options_window), "options_window", options_window);
  gtk_window_set_title (GTK_WINDOW (options_window), STR_RDJ_OPTIONS);
  gtk_window_set_position (GTK_WINDOW (options_window), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (options_window), FALSE, FALSE, FALSE);
  gtk_signal_connect (GTK_OBJECT (options_window), "destroy",
                      GTK_SIGNAL_FUNC (on_close_options),
                      options_window);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (options_window), vbox1);

  notebook1 = gtk_notebook_new ();
  gtk_widget_ref (notebook1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "notebook1", notebook1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (vbox1), notebook1, TRUE, TRUE, 0);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook1), GTK_POS_LEFT);

  // --------------------------------------------------------------------------------
  // start Options page
  frame2 = gtk_frame_new (STR_OPTIONS);
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_container_add (GTK_CONTAINER (notebook1), frame2);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 3);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame2), vbox2);

  frame3 = gtk_frame_new (STR_STARTUP);
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame3", frame3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox2), frame3, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame3), 3);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox3);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox3", vbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (frame3), vbox3);

  // save specific station
  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox2, FALSE, FALSE, 0);

  radiobutton1 = gtk_radio_button_new_with_label (startup_sound_group, STR_STATION);
  startup_sound_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton1));
  gtk_widget_ref (radiobutton1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "radiobutton1", radiobutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton1);
  gtk_box_pack_start (GTK_BOX (hbox2), radiobutton1, FALSE, FALSE, 0);
  gtk_widget_set_usize (radiobutton1, 100, -2);
  gtk_signal_connect (GTK_OBJECT (radiobutton1), "toggled",
                      GTK_SIGNAL_FUNC (on_station_radiobutton_toggled),
                      radiobutton1);

  // create from global station list
  win_options_menu = create_options_station_menu ();
  gtk_widget_ref (win_options_menu);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "win_options_menu", win_options_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (win_options_menu);
  gtk_box_pack_start (GTK_BOX (hbox2), win_options_menu, FALSE, FALSE, 0);
  if (options_station_choice_str)
    gtk_option_menu_set_history (GTK_OPTION_MENU (win_options_menu),
				 get_list_position (options_station_choice_str));
  if (options_station_toggle == TOGGLED)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton1), true);

  // save freq as default
  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox3, FALSE, FALSE, 0);

  radiobutton2 = gtk_radio_button_new_with_label (startup_sound_group, STR_FREQUENCY);
  startup_sound_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton2));
  gtk_widget_ref (radiobutton2);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "radiobutton2", radiobutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (hbox3), radiobutton2, FALSE, FALSE, 0);
  gtk_widget_set_usize (radiobutton2, 100, -2);
  gtk_signal_connect (GTK_OBJECT (radiobutton2), "toggled",
                      GTK_SIGNAL_FUNC (on_freqradiobutton_toggled), radiobutton2);
  if (options_freq_toggle == TOGGLED)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton2), true);

  options_freq_choice_entry = gtk_entry_new_with_max_length (MAX_FREQ_INPUT);
  gtk_widget_ref (options_freq_choice_entry);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "options_freq_choice_entry",
			    options_freq_choice_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (options_freq_choice_entry);
  gtk_box_pack_start (GTK_BOX (hbox3), options_freq_choice_entry, FALSE, FALSE, 0);
  gtk_widget_set_usize (options_freq_choice_entry, 50, -2);

  button1 = gtk_button_new_with_label (STR_LOAD_FREQ);
  gtk_box_pack_start (GTK_BOX (hbox3), button1, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (on_load_freq_clicked), NULL);
  gtk_widget_show (button1);

  // save last station as default
  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox3, FALSE, FALSE, 0);

  radiobutton2 = gtk_radio_button_new_with_label (startup_sound_group, STR_SLAD);
  startup_sound_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton2));
  gtk_widget_ref (radiobutton2);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "radiobutton2", radiobutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (hbox3), radiobutton2, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton2), "toggled",
                      GTK_SIGNAL_FUNC (on_slad_toggled), radiobutton2);
  if (options_slad == TOGGLED)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton2), true);
  //

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox4, FALSE, FALSE, 0);

  label6 = gtk_label_new (STR_TUNE_INC);
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox4), label6, FALSE, FALSE, 0);
  gtk_widget_set_usize (label6, 100, -2);

  options_tune_inc_entry = gtk_entry_new_with_max_length (MAX_FREQ_INPUT);
  gtk_widget_ref (options_tune_inc_entry);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "options_tune_inc_entry",
			    options_tune_inc_entry, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (options_tune_inc_entry);
  gtk_box_pack_start (GTK_BOX (hbox4), options_tune_inc_entry, FALSE, FALSE, 0);
  gtk_widget_set_usize (options_tune_inc_entry, 40, -2);

  // radio card frame
  frame4 = gtk_frame_new (STR_RADIO_CARD);
  gtk_widget_ref (frame4);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame4", frame4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (vbox2), frame4, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame4), 3);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox5);
  gtk_container_add (GTK_CONTAINER (frame4), hbox5);

  label7 = gtk_label_new ("  "STR_CHOOSE_CARD"  ");
  gtk_widget_ref (label7);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (hbox5), label7, FALSE, FALSE, 0);

  radio_optionmenu = gtk_option_menu_new ();
  gtk_widget_ref (radio_optionmenu);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "radio_optionmenu",
			    radio_optionmenu, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radio_optionmenu);
  gtk_box_pack_start (GTK_BOX (hbox5), radio_optionmenu, TRUE, FALSE, 0);
  radio_optionmenu_menu = gtk_menu_new ();

  // set the first menu item to user's option
  label1 = gtk_label_new (options_radio_card);
  menuitem = gtk_menu_item_new ();
  gtk_container_add (GTK_CONTAINER (menuitem), label1);
  gtk_widget_show (label1);
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (radio_optionmenu_menu), menuitem);
  gtk_signal_connect_object (GTK_OBJECT (menuitem), "activate",
			     GTK_SIGNAL_FUNC (on_radio_optionmenu_clicked),
			     GTK_OBJECT(label1));

  if (strcmp(options_radio_card, DFT_CARD) != 0) {
    label1 = gtk_label_new (DFT_CARD);
    menuitem = gtk_menu_item_new ();
    gtk_container_add (GTK_CONTAINER (menuitem), label1);
    gtk_widget_show (label1);
    gtk_widget_show (menuitem);
    gtk_menu_append (GTK_MENU (radio_optionmenu_menu), menuitem);
    gtk_signal_connect_object (GTK_OBJECT (menuitem), "activate",
			       GTK_SIGNAL_FUNC (on_radio_optionmenu_clicked),
			       GTK_OBJECT(label1));
  }

  // make the other three device menu items
  strcpy(buff, "/dev/radio0");
  for (i = 0; i < 4; i++) {
    if (strcmp(options_radio_card, buff) != 0) {
      label1 = gtk_label_new (buff);
      menuitem = gtk_menu_item_new ();
      gtk_container_add (GTK_CONTAINER (menuitem), label1);
      gtk_widget_show (label1);
      gtk_widget_show (menuitem);
      gtk_menu_append (GTK_MENU (radio_optionmenu_menu), menuitem);
      gtk_signal_connect_object (GTK_OBJECT (menuitem), "activate",
				 GTK_SIGNAL_FUNC (on_radio_optionmenu_clicked),
				 GTK_OBJECT(label1));
    }
    buff[strlen(buff)-1] = (char) (i + 49);
  }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (radio_optionmenu), radio_optionmenu_menu);

  frame8 = gtk_frame_new (STR_OTHER_PROGS);
  gtk_widget_ref (frame8);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame8", frame8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame8);
  gtk_box_pack_start (GTK_BOX (vbox2), frame8, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame8), 3);

  vbox8 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox8);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox8", vbox8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox8);
  gtk_container_add (GTK_CONTAINER (frame8), vbox8);

  hbox13 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox13);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox13", hbox13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox13);
  gtk_box_pack_start (GTK_BOX (vbox8), hbox13, FALSE, FALSE, 0);

  label1 = gtk_label_new ("  "STR_TV_PROG"  ");
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox13), label1, FALSE, FALSE, 0);

  options_tv_exec_entry = gtk_entry_new_with_max_length (MAX_STAT_INPUT);
  gtk_widget_ref (options_tv_exec_entry);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "options_tv_exec_entry",
			    options_tv_exec_entry, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (options_tv_exec_entry);
  gtk_box_pack_start (GTK_BOX (hbox13), options_tv_exec_entry, FALSE, FALSE, 0);
  gtk_widget_set_usize (options_tv_exec_entry, 120, -2);

  hbox13 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox13);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox13", hbox13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox13);
  gtk_box_pack_start (GTK_BOX (vbox8), hbox13, FALSE, FALSE, 0);

  label1 = gtk_label_new ("  "STR_MIXER_PROG"  ");
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox13), label1, FALSE, FALSE, 0);

  options_mixer_entry = gtk_entry_new_with_max_length (MAX_STAT_INPUT);
  gtk_widget_ref (options_mixer_entry);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "options_mixer_entry",
			    options_mixer_entry, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (options_mixer_entry);
  gtk_box_pack_start (GTK_BOX (hbox13), options_mixer_entry, FALSE, FALSE, 0);
  gtk_widget_set_usize (options_mixer_entry, 120, -2);

  // mute on exit
  checkbutton1 = gtk_check_button_new_with_label ("  "STR_MUTE_ON_EXIT"  ");
  gtk_widget_ref (checkbutton1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "checkbutton1", checkbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton1);
  gtk_box_pack_start (GTK_BOX (vbox2), checkbutton1, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (checkbutton1), "toggled",
                      GTK_SIGNAL_FUNC (on_mute_exit_checkbutton_toggled),
                      checkbutton1);
  if (options_mute_exit == TOGGLED) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton1), true);
    mute_on_exit = TOGGLED;
  }

  // wine on error
  checkbutton2 = gtk_check_button_new_with_label ("  "STR_PROMPT_ERR"  ");
  gtk_widget_ref (checkbutton2);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "checkbutton2", checkbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton2);
  gtk_box_pack_start (GTK_BOX (vbox2), checkbutton2, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (checkbutton2), "toggled",
                      GTK_SIGNAL_FUNC (on_wine_checkbutton_toggled),
                      checkbutton2);

  if (options_wine == TOGGLED) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton2), true);
    temp_wine = TOGGLED;
  }

  // keep location
  checkbutton1 = gtk_check_button_new_with_label ("  "STR_STORE_WIN_LOC"  ");
  gtk_widget_ref (checkbutton1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "checkbutton1", checkbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton1);
  gtk_box_pack_start (GTK_BOX (vbox2), checkbutton1, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (checkbutton1), "toggled",
                      GTK_SIGNAL_FUNC (on_store_location_checkbutton_toggled),
                      checkbutton1);

  if (options_store_location == TOGGLED) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton1), true);
    temp_store = TOGGLED;
  }

  label1 = gtk_label_new (STR_GENERAL);
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);
  // --------------------------------------------------------------------------------
  // edit stations page
  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox4);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox4", vbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox4);

  frame6 = gtk_frame_new (STR_STATION_LST);
  gtk_widget_ref (frame6);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame6", frame6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame6);
  gtk_box_pack_start (GTK_BOX (vbox4), frame6, TRUE, TRUE, 5);
  gtk_widget_set_usize (frame6, -2, 150);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_container_add (GTK_CONTAINER (frame6), scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  // columned list
  options_clist = gtk_clist_new (2);
  gtk_widget_ref (options_clist);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "options_clist", options_clist,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (options_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), options_clist);
  gtk_clist_set_column_width (GTK_CLIST (options_clist), 0, 52);
  gtk_clist_set_column_width (GTK_CLIST (options_clist), 1, 80);
  gtk_clist_column_titles_show (GTK_CLIST (options_clist));

  // fill clist with radio station info
  char station[MAX_STAT_INPUT+1], freq[MAX_FREQ_INPUT+1];
  tha_list->get_station_start();

  while(tha_list->get_station_next (station, freq) != RET_EOL) {
    gchar *szData[2] = {freq, station};
    gtk_clist_append (GTK_CLIST (options_clist), szData);
  }
  gtk_signal_connect (GTK_OBJECT (options_clist), "select_row",
		      GTK_SIGNAL_FUNC (on_clist_select_row), GTK_CLIST(options_clist));

  label8 = gtk_label_new (STR_FREQ_COL);
  gtk_widget_ref (label8);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label8);
  gtk_clist_set_column_widget (GTK_CLIST (options_clist), 0, label8);

  label9 = gtk_label_new (STR_STATION_COL);
  gtk_widget_ref (label9);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label9", label9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label9);
  gtk_clist_set_column_widget (GTK_CLIST (options_clist), 1, label9);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox5);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox5", vbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox5);
  gtk_box_pack_start (GTK_BOX (vbox4), vbox5, FALSE, FALSE, 0);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox6);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox6", hbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox6, FALSE, FALSE, 0);

  label10 = gtk_label_new (STR_STATION_NAME);
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label10", label10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_box_pack_start (GTK_BOX (hbox6), label10, FALSE, FALSE, 0);

  // new station entry
  options_new_station_entry = gtk_entry_new_with_max_length (MAX_STAT_INPUT);
  gtk_widget_ref (options_new_station_entry);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "options_new_station_entry",
			    options_new_station_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (options_new_station_entry);
  gtk_box_pack_start (GTK_BOX (hbox6), options_new_station_entry, FALSE, FALSE, 0);

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox7);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox7", hbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox7, FALSE, FALSE, 0);

  label11 = gtk_label_new (STR_FREQ_NUMBER);
  gtk_widget_ref (label11);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label11", label11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label11);
  gtk_box_pack_start (GTK_BOX (hbox7), label11, FALSE, FALSE, 0);
  gtk_widget_set_usize (label11, 100, -2);

  // new freq entry
  options_new_freq_entry = gtk_entry_new_with_max_length (MAX_FREQ_INPUT);
  gtk_widget_ref (options_new_freq_entry);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "options_new_freq_entry",
			    options_new_freq_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (options_new_freq_entry);
  gtk_box_pack_start (GTK_BOX (hbox7), options_new_freq_entry, FALSE, FALSE, 0);
  gtk_widget_set_usize (options_new_freq_entry, 50, -2);

  hbuttonbox4 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox4);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbuttonbox4", hbuttonbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox4);
  gtk_box_pack_start (GTK_BOX (vbox5), hbuttonbox4, FALSE, FALSE, 5);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox4), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox4), 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox4), 0, 0);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox4), 0, 0);

  // ADD station
  button6 = gtk_button_new_with_label (STR_ADD_LABEL);
  gtk_widget_ref (button6);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "button6", button6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button6);
  gtk_container_add (GTK_CONTAINER (hbuttonbox4), button6);
  GTK_WIDGET_SET_FLAGS (button6, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (button6), "clicked",
                      GTK_SIGNAL_FUNC (on_freqchart_add_clicked),
                      GTK_CLIST(options_clist));

  // UPDATE station
  button6 = gtk_button_new_with_label (STR_UPDATE_LABEL);
  gtk_widget_ref (button6);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "button6", button6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button6);
  gtk_container_add (GTK_CONTAINER (hbuttonbox4), button6);
  GTK_WIDGET_SET_FLAGS (button6, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (button6), "clicked",
                      GTK_SIGNAL_FUNC (on_freqchart_update_clicked),
                      &options_clist);

  // REMOVE station
  button7 = gtk_button_new_with_label (STR_REMOVE_LABEL);
  gtk_widget_ref (button7);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "button7", button7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button7);
  gtk_container_add (GTK_CONTAINER (hbuttonbox4), button7);
  GTK_WIDGET_SET_FLAGS (button7, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (button7), "clicked",
                      GTK_SIGNAL_FUNC (on_freqchart_remove_clicked),
                      &options_clist);

  frame5 = gtk_frame_new (STR_LIST_POSITION);
  gtk_widget_ref (frame5);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame5", frame5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox5), frame5, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame5), 3);

  hbuttonbox3 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox3);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbuttonbox3", hbuttonbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox3);
  gtk_container_add (GTK_CONTAINER (frame5), hbuttonbox3);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox3), GTK_BUTTONBOX_START);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox3), 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox3), 0, 0);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox3), 0, 0);

  button4 = gtk_button_new_with_label (STR_MOVE_UP);
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button4);
  gtk_container_add (GTK_CONTAINER (hbuttonbox3), button4);
  GTK_WIDGET_SET_FLAGS (button4, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (button4), "clicked",
                      GTK_SIGNAL_FUNC (on_freqchart_mv_up_clicked),
                      GTK_CLIST(options_clist));

  button5 = gtk_button_new_with_label (STR_MOVE_DOWN);
  gtk_widget_ref (button5);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "button5", button5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button5);
  gtk_container_add (GTK_CONTAINER (hbuttonbox3), button5);
  GTK_WIDGET_SET_FLAGS (button5, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (on_freqchart_mv_down_clicked),
                      GTK_CLIST(options_clist));

  label2 = gtk_label_new (STR_STATIONS_TAB);
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label2);

  // --------------------------------------------------------------------------------
  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox4);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox4", vbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox4);

  frame6 = gtk_frame_new (STR_STYLES);
  gtk_widget_ref (frame6);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame6", frame6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame6);
  gtk_box_pack_start (GTK_BOX (vbox4), frame6, FALSE, FALSE, 5);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_container_add (GTK_CONTAINER (frame6), hbox4);

  // selection font
  button1 = gtk_button_new_with_label ("  "STR_FONT_SELECTOR"  ");
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbox4), button1, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked", GTK_SIGNAL_FUNC (font_button_cb), NULL);

  font_entry = gtk_entry_new_with_max_length (25);
  gtk_widget_ref (font_entry);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "font_entry", font_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (font_entry);
  gtk_box_pack_start (GTK_BOX (hbox4), font_entry, TRUE, FALSE, 0);
  gtk_entry_set_editable (GTK_ENTRY (font_entry), false);
  gtk_entry_set_text (GTK_ENTRY (font_entry), use_font);
  gtk_widget_set_usize (font_entry, 175, -2);

  frame8 = gtk_frame_new (STR_COLORS);
  gtk_widget_ref (frame8);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame8", frame8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame8);
  gtk_box_pack_start (GTK_BOX (vbox4), frame8, FALSE, FALSE, 5);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox5);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox5", vbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox5);
  gtk_container_add (GTK_CONTAINER (frame8), vbox5);

  // box for button and darea
  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox5, FALSE, FALSE, 5);

  button1 = gtk_label_new ("  "STR_COLOR_FG"  ");
  gtk_widget_show (button1);
  gtk_widget_set_usize (button1, -2, 20);
  gtk_box_pack_start (GTK_BOX (hbox5), button1, FALSE, FALSE, 0);

  // foreground darea
  fg_cp = gnome_color_picker_new ();
  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (fg_cp), fg->red/255, fg->grn/255, fg->blu/255, 0);
  gtk_signal_connect (GTK_OBJECT (fg_cp), "color_set", GTK_SIGNAL_FUNC (color_set_fg_cb), NULL);
  gtk_box_pack_start (GTK_BOX (hbox5), fg_cp, FALSE, FALSE, 0);
  gtk_widget_show (fg_cp);

  // box for button and darea
  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox5, FALSE, FALSE, 5);

  button1 = gtk_label_new ("  "STR_COLOR_BG"  ");
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbox5), button1, FALSE, FALSE, 0);
  gtk_widget_set_usize (button1, -2, 20);

  // background darea
  bg_cp = gnome_color_picker_new ();
  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (bg_cp), bg->red/255, bg->grn/255, bg->blu/255, 0);
  gtk_signal_connect (GTK_OBJECT (bg_cp), "color_set", GTK_SIGNAL_FUNC (color_set_bg_cb), NULL);
  gtk_box_pack_start (GTK_BOX (hbox5), bg_cp, FALSE, FALSE, 0);
  gtk_widget_show (bg_cp);

  // new page (Styles)
  label2 = gtk_label_new (STR_STYLES);
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label2);

  // --------------------------------------------------------------------------------
  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox7);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox7", vbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox7);

  frame7 = gtk_frame_new (STR_SCAN_FREQS);
  gtk_widget_ref (frame7);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame7", frame7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame7);
  gtk_box_pack_start (GTK_BOX (vbox7), frame7, FALSE, FALSE, 15);
  gtk_container_set_border_width (GTK_CONTAINER (frame7), 3);

  vbox8 = gtk_vbox_new (FALSE, 3);
  gtk_widget_ref (vbox8);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "vbox8", vbox8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox8);
  gtk_container_add (GTK_CONTAINER (frame7), vbox8);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox9);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox9", hbox9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox9);
  gtk_box_pack_start (GTK_BOX (vbox8), hbox9, FALSE, FALSE, 0);

  label14 = gtk_label_new (STR_START_FREQ);
  gtk_widget_ref (label14);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label14", label14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label14);
  gtk_box_pack_start (GTK_BOX (hbox9), label14, FALSE, FALSE, 0);
  gtk_widget_set_usize (label14, 120, -2);

  options_scan_freq_start_entry = gtk_entry_new_with_max_length (MAX_FREQ_INPUT);
  gtk_widget_ref (options_scan_freq_start_entry);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "options_scan_freq_start_entry",
			    options_scan_freq_start_entry, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (options_scan_freq_start_entry);
  gtk_box_pack_start (GTK_BOX (hbox9), options_scan_freq_start_entry, FALSE, FALSE, 0);
  gtk_widget_set_usize (options_scan_freq_start_entry, 50, -2);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox10);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbox10", hbox10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox10);
  gtk_box_pack_start (GTK_BOX (vbox8), hbox10, FALSE, FALSE, 0);

  label15 = gtk_label_new (STR_STOP_FREQ);
  gtk_widget_ref (label15);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label15", label15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label15);
  gtk_box_pack_start (GTK_BOX (hbox10), label15, FALSE, FALSE, 0);
  gtk_widget_set_usize (label15, 120, -2);

  options_scan_freq_stop_entry = gtk_entry_new_with_max_length (MAX_FREQ_INPUT);
  gtk_widget_ref (options_scan_freq_stop_entry);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "options_scan_freq_stop_entry",
			    options_scan_freq_stop_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (options_scan_freq_stop_entry);
  gtk_box_pack_start (GTK_BOX (hbox10), options_scan_freq_stop_entry, FALSE, FALSE, 0);
  gtk_widget_set_usize (options_scan_freq_stop_entry, 50, -2);

  hbuttonbox2 = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox2);
  gtk_box_pack_start (GTK_BOX (vbox8), hbuttonbox2, FALSE, FALSE, 0);

  // scan button
  button8 = gtk_button_new_with_label (STR_SCAN_TAB);
  gtk_widget_ref (button8);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "button8", button8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button8);
  gtk_box_pack_start (GTK_BOX (hbuttonbox2), button8, FALSE, FALSE, 5);
  gtk_signal_connect (GTK_OBJECT (button8), "clicked",
                      GTK_SIGNAL_FUNC (on_scan_button_clicked), NULL);

  frame1 = gtk_frame_new (STR_INSTRUCTIONS);
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox7), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 3);

  label5 = gtk_label_new (STR_SCAN_DETAILS);
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_container_add (GTK_CONTAINER (frame1), label5);

  label3 = gtk_label_new (STR_SCAN_TAB);
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), 
			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 3), 
			      label3);
  // --------------------------------------------------------------------------------
  // About Page
  frame1 = gtk_frame_new (STR_RDJ_ABOUT);
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_container_add (GTK_CONTAINER (notebook1), frame1);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 3);

  // about box
  about_vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (about_vbox);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "about_vbox", about_vbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (about_vbox);
  gtk_container_add (GTK_CONTAINER (frame1), about_vbox);

  // rdj image
  icon_box = make_icon_button (options_window, PATH_RDJ);
  gtk_widget_ref (icon_box);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "icon_box", icon_box,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (icon_box);
  gtk_box_pack_start (GTK_BOX (about_vbox), icon_box, FALSE, TRUE, 15);

  // about message
  label5 = gtk_label_new (VERSIONSTR"\n\n(C) 2000-2001 David Mimms, Jr.\nfusion_dm@mailcity.com\n\nhttp://mimms.sf.net/\n\n"STR_UNDER_GPL);
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (about_vbox), label5, FALSE, FALSE, 0);

  label4 = gtk_label_new (STR_RDJ_ABOUT);
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 4), label4);
  // --------------------------------------------------------------------------------
  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, FALSE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox1), 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox1), 0, 0);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox1), 0, 0);

  // OK
  button1 = gtk_button_new_with_label (STR_OK);
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (on_options_save_clicked),
                      options_window);

  // APPLY
  button1 = gtk_button_new_with_label (STR_APPLY);
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (on_options_apply_clicked), NULL);

  button2 = gtk_button_new_with_label (STR_CLOSE);
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (options_window), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button2);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
                      GTK_SIGNAL_FUNC (on_close_options),
                      options_window);

  // load the entries
  gtk_entry_set_text (GTK_ENTRY (options_freq_choice_entry),     options_freq_choice);
  gtk_entry_set_text (GTK_ENTRY (options_tune_inc_entry),        options_tune_increment);
  gtk_entry_set_text (GTK_ENTRY (options_scan_freq_start_entry), options_scan_freq_start_str);
  gtk_entry_set_text (GTK_ENTRY (options_scan_freq_stop_entry),  options_scan_freq_stop_str);
  gtk_entry_set_text (GTK_ENTRY (options_tv_exec_entry),         options_tv_exec_str);
  gtk_entry_set_text (GTK_ENTRY (options_mixer_entry),           options_mixer_str);

  gtk_widget_show (options_window);
  return options_window;
}
