/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>

#include "main.h"
#include "defines.h"
#include "callbacks.h"
#include "interface.h"
#include "radioclass.h"
#include "scanclass.h"
#include "externs.h"

#ifdef I_HAVE_LIRC
# include <lirc/lirc_client.h>
static struct lirc_config *config;
#endif

#define debug 0

static gint input_tag = 0;

// ------------------------------------------------------------------
/*
  [commands]
  station up
  station down
  tune up
  tune down
  mute
  tv
  mixer
  dump lirc
  quit
*/
void
do_command (char *cmd)
{
  if (strcasecmp (cmd, "station up") == 0) {
    change_station_up ();
  } else if (strcasecmp (cmd, "station down") == 0) {
    change_station_down ();
  } else if (strcasecmp (cmd, "tune up") == 0) {
    on_freq_inc_clicked (NULL, NULL);
  } else if (strcasecmp (cmd, "tune down") == 0) {
    on_freq_dec_clicked (NULL, NULL);
  } else if (strcasecmp (cmd, "mute") == 0) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_mute_btn),
				  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle_mute_btn)) ? false : true);
  } else if (strcasecmp (cmd, "tv") == 0) {
    if (options_tv_exec_str[0])
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_tv_btn),
				    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle_tv_btn)) ? false : true);
  } else if (strcasecmp (cmd, "mixer") == 0) {
    on_mixer_clicked (NULL, NULL);
  } else if (strcasecmp (cmd, "dump lirc") == 0) {
    gdk_input_remove (input_tag);
#ifdef I_HAVE_LIRC
    lirc_freeconfig (config);
    lirc_deinit ();
#endif
    lirc_works = false;
    TUNE (current_freq); // update the freq display
  } else if (strcasecmp (cmd, "quit") == 0) {
    ExitApp ();
  } else {
    printf ("[rdj] unrecognized cmd: \"%s\"\n", cmd);
  }
}
// ------------------------------------------------------------------
int
rdj_lirc_init ()
{
#ifdef I_HAVE_LIRC
  int fd;
  
  if ((fd = lirc_init ("rdj", debug)) == -1) {
    fprintf (stderr, "[rdj] infrared remote support is NOT available.\n");
    return -1;
  }
  
  if (lirc_readconfig (NULL, &config, NULL) != 0) {
    fprintf (stderr, "[rdj] lirc config returned an error.\n");
    lirc_deinit ();
    return -1;
  }
  
  fcntl (fd, F_SETFL, O_NONBLOCK);
  fcntl (fd, F_SETFD, FD_CLOEXEC);
  
  return fd;
#else
  return -1;
#endif
}
// ------------------------------------------------------------------
void
rdj_lirc_havedata (void *, int, GdkInputCondition)
{
#ifdef I_HAVE_LIRC
  char *code, *cmd;
  int ret = -1;
  
  while (lirc_nextcode (&code) == 0 && code != NULL) {
    ret = 0;
    while (lirc_code2char (config, code, &cmd) == 0 && cmd != NULL) {
      if (debug) fprintf (stderr,"[rdj] cmd: \"%s\"\n", cmd);
      do_command (cmd);
    }
    free (code);
  }

  // on LIRC error, shutdown added input
  if (ret == -1) {
    gdk_input_remove (input_tag);
    lirc_freeconfig (config);
    lirc_deinit ();
    lirc_works = false;
    TUNE (current_freq); // update the freq display
  }
#endif
}
// ------------------------------------------------------------------
void
enable_lirc_cb ()
{
  if (lirc_works) return;

  int fd;

  if ((fd = rdj_lirc_init ()) == -1) return;

  input_tag = gdk_input_add (fd, GDK_INPUT_READ, rdj_lirc_havedata, NULL);

  lirc_works = true;
}
