/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <gtk/gtk.h>
#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "main.h"
#include "defines.h"
#include "callbacks.h"
#include "interface.h"
#include "radioclass.h"
#include "stationlist.h"
#include "config.h"
#include "externs.h"
#include "lirc.h"

//------------------------------------------------------------------------
void
handle_sig (int sig)
{
  ExitApp();
}
//------------------------------------------------------------------------
void
set_location ()
{
  char x[100] = "0", y[100] = "0", str[100];
  FILE *fp;

  snprintf (str, sizeof (str), "%s/"Cfg_location_file, getenv ("HOME"));

  if ((fp = fopen (str, "r")) == NULL) return;

  fgets (x, sizeof (x), fp);
  fgets (y, sizeof (y), fp);
  fclose (fp);

  gdk_window_move (win_main->window, atoi (x), atoi (y));
}
//------------------------------------------------------------------------
void
set_colorz ()
{
  if (freq_fg_gc) gdk_gc_destroy (freq_fg_gc);
  if (freq_bg_gc) gdk_gc_destroy (freq_bg_gc);

  if (freqdisp_fg) g_free (freqdisp_fg);
  if (freqdisp_bg) g_free (freqdisp_bg);

  freqdisp_fg = (GdkColor *) g_malloc (sizeof (GdkColor));
  freqdisp_bg = (GdkColor *) g_malloc (sizeof (GdkColor));
  
  freqdisp_fg->red   = fg->red;
  freqdisp_fg->green = fg->grn;
  freqdisp_fg->blue  = fg->blu;
  gdk_color_alloc (gdk_colormap_get_system (), freqdisp_fg);

  freqdisp_bg->red   = bg->red;
  freqdisp_bg->green = bg->grn;
  freqdisp_bg->blue  = bg->blu;
  gdk_color_alloc (gdk_colormap_get_system (), freqdisp_bg);

  if (win_made) {
    freq_fg_gc = gdk_gc_new (win_main->window);
    gdk_gc_set_foreground (freq_fg_gc, freqdisp_fg);
    
    freq_bg_gc = gdk_gc_new (win_main->window);
    gdk_gc_set_foreground (freq_bg_gc, freqdisp_bg);
    
    SetTextFreq (current_freq);
  }
}

void
win_shake (const int x, const int y)
{
  bool swap = false;
  FILE *fp;
  
  char file[100];
  sprintf (file, "%s/"Cfg_shake_file, getenv("HOME"));
  
  if ((fp = fopen (file, "r"))) {
    fclose (fp);
    return;
  }
  
  fp = fopen (file, "w");
  fclose (fp);
  
  for (int i = 0; i < 250; i++) {
    swap = (swap == true) ? false : true;
    if (swap) gdk_window_move (win_main->window, x+15, y);
    else gdk_window_move (win_main->window, x-30, y);
  }
}
//------------------------------------------------------------------------
int
main(int argc, char *argv[])
{
  char station[MAX_STAT_INPUT+1], freq[MAX_FREQ_INPUT+1];
  int ret;

  signal (SIGHUP,  SIG_IGN);
  signal (SIGINT,  handle_sig);
  signal (SIGQUIT, handle_sig);
  signal (SIGTERM, handle_sig);
  signal (SIGPIPE, handle_sig);

  gtk_set_locale();

  gtk_init (&argc, &argv);

  tha_list = &tha_list_;

  strcpy (current_freq, FREQLOW);
  strcpy (freq, "");

  // read the configuration first
  ReadConfig ();

  // set the frequency display's colors
  set_colorz ();

  // if no card is specified, insert default
  if (strlen (options_radio_card) == 0) {
    strcpy (options_radio_card, DFT_CARD);
  } else {
    if (tha_dj.open_card (options_radio_card) == RET_CARD) {
      printf ("[rdj] error opening card\n");
    }
  }

  // start check for a station to use from config file
  // there is a default station, get it's freq
  if (strlen (options_station_choice_str) != 0) {
    tha_list->get_station_start ();
    while (tha_list->get_station_next(station, freq) != RET_EOL) {
      if (strcmp(station, options_station_choice_str) == 0) {
	options_station_toggle = TOGGLED;
	options_freq_toggle    = UNTOGGLED;
	break;
      }
    }
    // if the default station isn't in the list
    if (options_station_toggle != TOGGLED) {
      options_station_toggle = UNTOGGLED;
      options_freq_toggle    = TOGGLED;
    }
  } else { // no default station given
    options_station_toggle = UNTOGGLED;
    options_freq_toggle    = TOGGLED;
  }

  // no default station frequency and there is a default frequency
  if (options_station_toggle == UNTOGGLED && strlen (options_freq_choice) != 0) {
    strcpy (freq, options_freq_choice);
  }

  // display win before proceeding
  win_main = create_win_main ();
  win_made = TOGGLED;

  // start LIRC support
  enable_lirc_cb ();

  if (options_store_location) set_location ();
  
  // after all that, tell 'em what's up!
  if (strlen(freq) == 0) {
    strcpy(freq, FREQLOW);
    SetTextFreq(freq);
    
    on_options_clicked (NULL, NULL);
    gtk_notebook_set_page (GTK_NOTEBOOK (notebook1), PAGE_SCANS);
    MessageBox(STR_NEED_DFT_STATION);
  }
  
  // turn on the radio, save ret for next if
  if ((ret = tha_dj.tune(freq)) == RET_ERR) { return 1; }
  
  // if couldn't turn the card on
  if (ret == RET_CARD && options_wine == TOGGLED) {
    if (options_win_status != TOGGLED) {
      on_options_clicked (NULL, NULL);
      gtk_notebook_set_page (GTK_NOTEBOOK (notebook1), PAGE_OPTNS);
    }
    MessageBox (STR_NO_OPEN_CARD);
  }
  
  TUNE (freq);
  gtk_main();
  return 0;
}
//------------------------------------------------------------------------
gint
ExitApp()
{
  if (options_mute_exit == TOGGLED) tha_dj.mute();

  static int x, y;

  // save location
  if (options_store_location) {
    char str[100];
    FILE *fp;
    
    snprintf (str, sizeof (str), "%s/"Cfg_location_file, getenv ("HOME"));
    
    if ((fp = fopen (str, "w"))) {
      gdk_window_get_position (win_main->window, &x, &y);
      fprintf (fp, "%i\n%i\n", x, y);
      fclose (fp);
    }
  }

  // save last station as default
  if (options_slad == TOGGLED) {
    tha_list->free_list ();
    ReadConfig ();
    save_options (0);
  }

  win_shake (x, y);

  printf("\n\t"VERSIONSTR", Copyright (C) 2000, 2001 David Mimms, Jr.\n\trdj"
" comes with ABSOLUTELY NO WARRANTY; type 'show w' for details.\n\tThis is free software,"
" and you are welcome to redistribute it\n\tunder certain conditions; type `show c' for details.\n"
"\n\tPlease visit: http://mimms.sourceforge.net/ (homepage)\n");

  gtk_main_quit();
  return FALSE;
}
