/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// radio class imp
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <asm/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <linux/videodev.h>
#include <errno.h>

#include "defines.h"
#include "radioclass.h"

#ifndef major
# define major(dev)  (((dev) >> 8) & 0xff)
#endif

#define USE_MAJOR 81

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
radio_cs::
radio_cs ()
{
  strncpy (rcard, DFT_CARD, 256);
  strcpy (main_freq, FREQLOW);
  tunevol = HALFMAXVOL;
  mute_on = NO;
  fd = 0;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
radio_cs::
radio_cs (char *cd)
{
  strncpy (rcard, cd, 256);
  strcpy (main_freq, FREQLOW);
  tunevol = HALFMAXVOL;
  mute_on = NO;
  fd = 0;
  open_card (rcard);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
radio_cs::
radio_cs (char *cd, char *fq)
{
  strncpy (rcard, cd, 256);
  mute_on = NO;
  tunevol = HALFMAXVOL;
  fd = 0;
  tune (fq);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int radio_cs::
tune (char *fq)
{
#ifdef DEBUG
  printf ("::radio_cs:tune << start fq=%s\n", fq);
#endif
  ulong freq;

  if (!fd) {
    // use the card specified in the constructor
    if (open_card (rcard) == RET_CARD) {
#ifdef DEBUG
      printf ("::radio_cs:tune << no card\n");
#endif
      return RET_CARD;
    }
  }

  strcpy (main_freq, fq);

  freq = (ulong) ceil(atof(fq) * fact);
  
  if ((freq < vt.rangelow) || (freq > vt.rangehigh)) {
#ifdef DEBUG
    printf ("::radio_cs:tune << frequency %2.1f out of range (%2.1f - %2.1f MHz)\n",
	     (freq / fact), (vt.rangelow / fact), 
	     (vt.rangehigh / fact));
    printf ("::radio_cs:tune << ret rng\n");
#endif

    sprintf (freq_range, "freq: %2.1f!!   [%2.1f - %2.1f MHz]", (freq / fact), (vt.rangelow / fact), (vt.rangehigh / fact));
    return RET_RNG;
  }
  
  // frequency sanity checked, proceed
  if (ioctl (fd, VIDIOCSFREQ, &freq) < 0) {
#ifdef DEBUG
    printf ("::radio_cs:tune << ret err << set freq: %s\n", strerror (errno));
#endif
    return RET_ERR;
  }
  
  va.audio = 0;
  va.flags = 0;
  va.mode = VIDEO_SOUND_STEREO;

  if (mute_on == NO) {
    // set the volume
    ioctl (fd, VIDIOCSAUDIO, &va);
  }
  
  strcpy (main_freq, fq);
#ifdef DEBUG
  printf ("::radio_cs:tune << return ok\n");
#endif
  return RET_OK;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int radio_cs::
mute ()
{
#ifdef DEBUG
  printf ("::radio_cs:mute\n");
#endif
  if (!fd) return RET_CARD;

  va.audio = 0;
  va.volume = 0;
  va.flags = VIDEO_AUDIO_MUTE;
  if (ioctl (fd, VIDIOCSAUDIO, &va) == -1)
    printf ("::radio_cs:mute << %s\n", strerror (errno));

  mute_on = YES;
  return RET_OK;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int radio_cs::
unmute ()
{
#ifdef DEBUG
  printf ("::radio_cs:unmute\n");
#endif
  if (!fd) return RET_CARD;

  va.audio = 0;
  va.volume = tunevol;
  va.flags = 0;
  if (ioctl (fd, VIDIOCSAUDIO, &va) == -1)
    printf ("::radio_cs:unmute << %s\n", strerror (errno));

  mute_on = NO;
  return RET_OK;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int radio_cs::
incfreq (char *inc)
{
  if (!fd) return RET_CARD;

  double newfreq = atof (main_freq) + atof (inc);
  char cnew[15];
  sprintf (cnew,"%2.2f", newfreq);
  return tune (cnew);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int radio_cs::
decfreq (char *dec)
{
  if (!fd) return RET_CARD;

  double newfreq = atof (main_freq) - atof (dec);
  char cnew[15];
  sprintf (cnew,"%2.2f", newfreq);
  return tune (cnew);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int radio_cs::
load_mods (char *card, int mayor)
{
  static bool done = false;
  int fp;

  // just opening dev/video loads the modules
  // only have to do this once
  if (done == false) {
    fp = open ("/dev/video", O_RDWR);
    if (fp) close (fp);
  }

  done = true;

  fp = open (card, O_RDONLY);
  return fp;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int radio_cs::
open_card (char *card)
{
#ifdef DEBUG
  printf ("::radio_cs:open_card << start\n");
#endif
  if (!card[0]) return RET_ERR;

  int ret;

  if (fd) close (fd);

  // open the device
  if ((fd = load_mods (card, USE_MAJOR)) == -1) {
    printf ("::radio_cs:open_card << can't open %s: %s\n", card, strerror (errno));
    fd = 0;
    return RET_CARD;
  }

#ifdef DEBUG
  printf ("::radio_cs:open_card << card = %s\n", card);
#endif

  vt.tuner = 0;

  if (ioctl(fd, VIDIOCSTUNER, &vt) == -1) {
    printf ("::radio_cs:open_card << set freq: %s\n", strerror (errno));
    return RET_ERR;
  }

  // get frequency range
  ret = ioctl(fd, VIDIOCGTUNER, &vt);
  if (ret == -1 || (vt.flags & VIDEO_TUNER_LOW) == 0)
    fact = 16.;
  else
    fact = 16000.;

  // mute the radio before trying to tune
  va.audio = 0;
  va.volume = 0;
  va.flags = VIDEO_AUDIO_MUTE;
  ioctl (fd, VIDIOCSAUDIO, &va);

  if (mute_on == NO) {
    // set the volume
    ioctl (fd, VIDIOCSAUDIO, &va);
  }

  sprintf (freq_range, "(%2.1f - %2.1f MHz)", (vt.rangelow / fact), (vt.rangehigh / fact));

#ifdef DEBUG
  printf ("::radio_cs:open_card << return ok\n");
#endif
  return RET_OK;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int radio_cs::
get_freq (char *fq)
{
#ifdef DEBUG
  printf ("::radio_cs:get_freq\n");
#endif

  strcpy(fq, main_freq);
  return RET_OK;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
unsigned long radio_cs::
get_sig ()
{
  if (!fd) return 0;

  if (ioctl (fd, VIDIOCGTUNER, &vt) == -1) return 0;

  vt.tuner = 0;
  ioctl (fd, VIDIOCGTUNER, &vt);
  return vt.signal;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void radio_cs::
off (int m)
{
#ifdef DEBUG
  printf ("::radio_cs:off << mute=%i\n", m);
#endif

  if (m) {
    va.audio = 0;
    va.volume = 0;
    va.flags = VIDEO_AUDIO_MUTE;
    ioctl (fd, VIDIOCSAUDIO, &va);
  }

  if (fd) close (fd);
  fd = 0;
}
