/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <locale.h>
#include "defines.h"
#include "config.h"
#include "radioclass.h"

// --------------------------------------------------------------
// specify the left, and get the right (from the config file)
void
get_info (char *spec, char *ret)
{
  FILE *fp;
  char path[256], left[128], right[128];
  
  sprintf (path, "%s/"CfgFile, getenv("HOME"));
  
  if ((fp = fopen (path, "r")) == NULL) {
    printf ("[rdj_term] can't open file \"%s\": %s\n",
	    path, strerror (errno));
    exit (0);
  } else {
    while (fgets (path, sizeof (path), fp)) {
      SplitEquation (path, left, right);
      if (strcasecmp (left, spec) == 0) {
	strcpy (ret, right);
	break;
      }
    }
    fclose (fp);
  }
}
// --------------------------------------------------------------
// args: [-c card] [-off] freq | station name
// none is /dev/radio and default station from ~/.rdj
int
main (int argc, char **argv)
{
  radio_cs rdj;
  char station[MAX_STAT_INPUT+1], fq[MAX_FREQ_INPUT+1], 
  	   temp[MAX_STAT_INPUT+1], card[128];
  bool roff = false;

  setlocale(LC_ALL, "");

  if (argc != 4)
    get_info (CFG_card, card);                      // get the configured card

  if (argc == 1) {                                // no args, tune dft station wit
    get_info (CFG_station, station);              // the default card
    get_info (station, fq);

    if (strlen (fq)) {
      printf ("Tuning station: %s @ %s with %s\n", station, fq, card);
      rdj.open_card (card);
      rdj.tune (fq);
    }
  } else {
    for (int i = 1; i < argc; i++) {              // the options
      if (strcmp (argv[i], "-c") == 0) {          // -c device
	if (argv[i+1]) strncpy (card, argv[++i], sizeof (card));
	else printf ("[rdj_term] You need to specify a card with this option!\n");
      } else if (strcmp (argv[i], "-off") == 0) { // -off
	roff = true;
	break;
      } else strncpy (temp, argv[i], sizeof(temp));   // the freq or station name
    }

    if (roff) {                                   // turn off the radio
      rdj.open_card (card);                       // to mute, must open card first
      rdj.mute ();
    } else if (strlen (temp)) {                     // user specified a station by freq
      if (rdj.tune (temp) != RET_OK) {              // or name
	get_info (temp, station);                   // retrieve frequency from config
	printf ("Tuning station: %s @ %s with %s\n", temp, station, card);
	rdj.tune (station);                       // then tune it
      }
    } else {                                      // user specified a card but no freq
      get_info (CFG_station, station);            // so tune default station
      get_info (station, fq);
      if (strlen (fq)) {
	printf ("Trying station: %s @ %s with %s\n", station, fq, card);
	printf ("If the card you selected won't open, rdj_term will use /dev/radio.\n");
	rdj.tune (fq);                            // auto opens card
      }
    }
  }
  return 0;
}
