/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <math.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <asm/types.h>
#include <sys/ioctl.h>
#include <linux/videodev.h>
#include <string.h>
#include <errno.h>

#include "scanclass.h"

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
scanr::
scanr ()
{
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
scanr::
scanr (char a[], char b[], char c[], char d[], stations_list_cs &l)
{
  scan (a, b, c, d, l);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int scanr::
scan (char card[], char cstart[], char cend[], char cinc[], stations_list_cs &lst)
{
  // mostly modified from Russell Kroll's fmtools
  int fd, ret, i, tries = 25, counter = 1;
  struct video_tuner vt;
  float	perc, begval, incval, endval;
  long	lowf, highf, freq, totsig, incr, fact, tempf;
  char buff[100], buff2[100];

  if ((fd = open (card, O_RDONLY)) == -1) {
    printf("::scanr:scan << unable to open %s: %s\n", card, strerror (errno));
    return RET_CARD;
  }

  incval = atof (cinc);
  endval = atof (cend);
  begval = atof (cstart);

  vt.tuner = 0;
  if ((ret = ioctl(fd, VIDIOCGTUNER, &vt)) == -1) {
    printf("::scanr:scan << unable to set tuner: %s\n", strerror (errno));
    close (fd);
    return RET_CARD;
  }

  if ((vt.flags & VIDEO_TUNER_LOW) == 0)
    fact = 16;
  else
    fact = 16000;
  
  lowf = (long) (fact * (ceil(rint(begval * 10)) / 10));
  highf = (long) (fact * (ceil(rint(endval * 10)) / 10));
  incr  = (long) (fact * incval);

  // allow scanning forwards and backwards
  if (lowf > highf) {
    tempf = lowf;
    lowf = highf;
    highf = tempf;
  }

  for (freq = lowf; freq <= highf; freq += incr) {
    ioctl (fd, VIDIOCSFREQ, &freq);
    
    // lock on
    usleep (400000);
	
    totsig = 0;
    for (i = 0; i < tries; i++) {
      vt.tuner = 0;
      // get the info
      ret = ioctl (fd, VIDIOCGTUNER, &vt);
      totsig += vt.signal;
      usleep (15000);
    }

    perc = (totsig / (65535.0 * tries));

    if (perc > .5) {
      sprintf (buff, "%3.1f", (freq / (double) fact));
      sprintf (buff2, "New Scan %i", counter++);

      if (lst.add_station (buff2, buff) == RET_MEM) {
	printf ("::scanr:scan << not enough memory scan list\n");
  	break;
      }
    }
  }
  
  close (fd);

  return RET_OK;
}
