/*
    rdj - a gtk+ radio program for video4linux devices
    Copyright (C) 2000-2001  David Mimms, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// station list implementation

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "stationlist.h"

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
stations_cs::
stations_cs ()
{
  strcpy(station, "");
  strcpy(freq,    "");
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
stations_cs::
stations_cs (const char st[MAX_STAT_INPUT+1],
	     const char fq[MAX_FREQ_INPUT+1])
{
  strcpy(station, st);
  strcpy(freq,    fq);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void stations_cs::
set_info (const char st[MAX_STAT_INPUT+1],
	  const char fq[MAX_FREQ_INPUT+1])
{
  strcpy(station, st);
  strcpy(freq,    fq);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void stations_cs::
get_info (char *st, char *fq)
{
  strcpy(st, station);
  strcpy(fq, freq);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  STATION LIST CLASS
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
stations_list_cs::
stations_list_cs()
{
  NEXTITEM = HEAD = CURRENT = &LISTHEAD;
  can_free = NO;
  total_recs = 0;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
stations_list_cs::
stations_list_cs (const char st[MAX_STAT_INPUT+1],
		  const char fq[MAX_FREQ_INPUT+1])
{
  NEXTITEM = HEAD = CURRENT = &LISTHEAD;
  can_free = NO;
  total_recs = 0;
  add_station(st, fq);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
stations_list_cs::
~stations_list_cs ()
{
  free_list();
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int stations_list_cs::
add_station (const char st[MAX_STAT_INPUT+1],
	     const char fq[MAX_FREQ_INPUT+1])
{
  stations_cs *temp;

  if ((temp = create_station(CURRENT)) != NULL) {
    CURRENT = temp;
    CURRENT->set_info(st, fq);
  } else {
    return RET_MEM;
  }

  can_free = YES;
  total_recs++;
  return RET_OK;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
stations_cs *stations_list_cs::
create_station (stations_cs *crnt)
{
  stations_cs *add;
  add = (stations_cs *) malloc(sizeof(stations_cs));
  if (add != NULL) {
    crnt->next = add;
    add->next  = NULL;
  }
  return add;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void stations_list_cs::
free_list ()
{
  if (can_free == NO) return;

  total_recs = 0;

  free_loop(HEAD->next);

  NEXTITEM = HEAD = CURRENT = &LISTHEAD;
  can_free = NO;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void stations_list_cs::
free_loop (stations_cs *ptr)
{
  if (ptr->next != NULL)
    free_loop(ptr->next);
  free(ptr);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int stations_list_cs::
get_station_start ()
{
  NEXTITEM = HEAD;

  if (total_recs == 0) return 0;
  else return 1;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int stations_list_cs::
get_station_next (char *st, char *fq)
{
  if (NEXTITEM->next == NULL) return RET_EOL;

  NEXTITEM = NEXTITEM->next;
  NEXTITEM->get_info(st, fq);

  return RET_OK;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int stations_list_cs::
get_num_recs ()
{
  return total_recs;
}
